Package: spicyR
Type: Package
Title: Spatial analysis of in situ cytometry data
Version: 1.22.0
Authors@R: c(
    person("Nicolas", "Canete", , "nicolas.canete@sydney.edu.au", role = "aut"),
    person("Ellis", "Patrick", , "ellis.patrick@sydney.edu.au", role = c("aut", "cre")),
    person("Nicholas", "Robertson", , "nicholas.robertson@sydney.edu.au", role = "ctb"),
    person("Alex", "Qin", , "alex.qin@sydney.edu.au", role = "ctb"),
    person("Farhan", "Ameen", ,"farhan.ameen@sydney.edu.au", role = "ctb"),
    person("Shreya", "Rao", ,"shreya.rajeshrao@sydney.edu.au", role = "ctb")
  )
Description: The spicyR package provides a framework for performing inference on changes in spatial relationships between pairs of cell types for cell-resolution spatial omics technologies. spicyR consists of three primary steps: (i) summarizing the degree of spatial localization between pairs of cell types for each image; (ii) modelling the variability in localization summary statistics as a function of cell counts and (iii) testing for changes in spatial localizations associated with a response variable.  
License: GPL (>=2)
LazyData: true
biocViews: SingleCell, CellBasedAssays, Spatial
Encoding: UTF-8
Depends: R (>= 4.1)
VignetteBuilder: knitr
BugReports: https://github.com/SydneyBioX/spicyR/issues
URL: https://ellispatrick.github.io/spicyR/
        https://github.com/SydneyBioX/spicyR,
        https://sydneybiox.github.io/spicyR/
Imports: BiocParallel, ClassifyR, S4Vectors, SingleCellExperiment,
        SpatialExperiment, SummarizedExperiment, cli, concaveman,
        coxme, data.table, dplyr, ggforce, ggh4x, ggnewscale, ggplot2,
        ggthemes, grDevices, lifecycle, lmerTest, magrittr, methods,
        pheatmap, rlang, scales, scam, simpleSeg, spatstat.explore,
        spatstat.geom, stats, survival, tibble, tidyr
Suggests: SpatialDatasets, BiocStyle, knitr, rmarkdown, pkgdown,
        imcRtools, testthat (>= 3.0.0)
RoxygenNote: 7.3.3
Config/testthat/edition: 3
git_url: https://git.bioconductor.org/packages/spicyR
git_branch: RELEASE_3_22
git_last_commit: a906b18
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 06:46:40 UTC; biocbuild
Author: Nicolas Canete [aut],
  Ellis Patrick [aut, cre],
  Nicholas Robertson [ctb],
  Alex Qin [ctb],
  Farhan Ameen [ctb],
  Shreya Rao [ctb]
Maintainer: Ellis Patrick <ellis.patrick@sydney.edu.au>

# Generated by roxygen2: do not edit by hand

export(bind)
export(colTest)
export(convPairs)
export(getPairwise)
export(getProp)
export(imageCrossPlot)
export(plotImage)
export(signifPlot)
export(spicy)
export(spicyBoxPlot)
export(topPairs)
exportClasses(SpicyResults)
import(SpatialExperiment)
import(SummarizedExperiment)
import(dplyr)
import(ggh4x)
import(ggplot2)
import(ggthemes)
importFrom(BiocParallel,MulticoreParam)
importFrom(BiocParallel,SerialParam)
importFrom(BiocParallel,bplapply)
importFrom(BiocParallel,bpmapply)
importFrom(ClassifyR,colCoxTests)
importFrom(S4Vectors,as.data.frame)
importFrom(SingleCellExperiment,SingleCellExperiment)
importFrom(SingleCellExperiment,colData)
importFrom(SingleCellExperiment,colPair)
importFrom(SpatialExperiment,spatialCoords)
importFrom(SummarizedExperiment,colData)
importFrom(cli,cli_abort)
importFrom(cli,cli_inform)
importFrom(concaveman,concaveman)
importFrom(coxme,coxme)
importFrom(data.table,":=")
importFrom(data.table,.SD)
importFrom(data.table,CJ)
importFrom(data.table,as.data.table)
importFrom(data.table,setkey)
importFrom(dplyr,across)
importFrom(dplyr,any_of)
importFrom(dplyr,arrange)
importFrom(dplyr,bind_rows)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,rename)
importFrom(dplyr,select)
importFrom(dplyr,ungroup)
importFrom(dplyr,where)
importFrom(ggforce,geom_arc_bar)
importFrom(ggforce,geom_circle)
importFrom(ggnewscale,new_scale)
importFrom(ggplot2,aes)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_text)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,guide_axis)
importFrom(ggplot2,guide_colourbar)
importFrom(ggplot2,guide_legend)
importFrom(ggplot2,guides)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_colour_gradient2)
importFrom(ggplot2,scale_shape_manual)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_classic)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,colors)
importFrom(grid,gpar)
importFrom(grid,grobTree)
importFrom(grid,polygonGrob)
importFrom(lifecycle,deprecate_soft)
importFrom(lifecycle,deprecate_warn)
importFrom(lmerTest,lmer)
importFrom(magrittr,"%>%")
importFrom(methods,is)
importFrom(pheatmap,pheatmap)
importFrom(rlang,.data)
importFrom(scales,squish)
importFrom(scam,scam)
importFrom(spatstat.explore,density.ppp)
importFrom(spatstat.geom,area)
importFrom(spatstat.geom,border)
importFrom(spatstat.geom,closepairs)
importFrom(spatstat.geom,convexhull)
importFrom(spatstat.geom,discs)
importFrom(spatstat.geom,inside.owin)
importFrom(spatstat.geom,intersect.owin)
importFrom(spatstat.geom,nearest.valid.pixel)
importFrom(spatstat.geom,owin)
importFrom(spatstat.geom,ppp)
importFrom(spatstat.geom,solapply)
importFrom(spatstat.geom,union.owin)
importFrom(stats,lm)
importFrom(stats,p.adjust)
importFrom(stats,predict)
importFrom(stats,quantile)
importFrom(stats,setNames)
importFrom(stats,t.test)
importFrom(stats,weights)
importFrom(stats,wilcox.test)
importFrom(survival,coxph)
importFrom(tibble,column_to_rownames)
importFrom(tibble,rownames_to_column)
importFrom(tidyr,pivot_longer)
importFrom(tidyr,separate)

# spicyR

<img src="https://raw.githubusercontent.com/ellispatrick/spicyR/master/inst/spicyR.png" align="right" width="200" />

Spatial analysis of in situ cytometry data.

## Overview


The **spicyR** package provides a framework for performing inference on changes in spatial relationships between pairs of cell types for cell-resolution spatial omics technologies. spicyR consists of three primary steps: (i) summarizing the degree of spatial localization between pairs of cell types for each image; (ii) modelling the variability in localization summary statistics as a function of cell counts and (iii) testing for changes in spatial localizations associated with a response variable. 


## Installation

For the Bioconductor release version, run the following.
```r
if (!require("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("spicyR")
```

If you would like the most up-to-date features, install the most recent development version.
```r
# Install the development version from Bioconductor:
if (!require("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

# The following initializes usage of Bioc devel
# This will update all your Bioconductor packages to devel version
BiocManager::install(version='devel')

BiocManager::install("spicyR")

# Otherwise install the development version from GitHub:
# install.packages("devtools")
devtools::install_github("ellispatrick/spicyR")
library(spicyR)
```

## Submitting an issue or feature request

`spicyR` is still under active development. We would greatly appreciate any and 
all feedback related to the package.

* R package related issues should be raised [here](https://github.com/ellispatrick/spicyR/issues).
* For general questions and feedback, please contact us directly via [ellis.patrick@sydney.edu.au](mailto:ellis.patrick@sydney.edu.au).


## Authors

* **Nicolas Canete**
* **Ellis Patrick**  - [@TheEllisPatrick](https://twitter.com/TheEllisPatrick)

## Citation

<div class="oxford-citation-text">

Nicolas P Canete, Sourish S Iyengar, John T Ormerod, Heeva Baharlou, Andrew N Harman, Ellis Patrick, spicyR: spatial analysis of _in situ_ cytometry data in R, _Bioinformatics_, Volume 38, Issue 11, 1 June 2022, Pages 3099–3105, [https://doi.org/10.1093/bioinformatics/btac268](https://doi.org/10.1093/bioinformatics/btac268)

</div>

#' Pairwise cross-plot of spatial associations
#'
#' Builds a pairwise scatterplot of marker-marker associations for a single image
#' using the output of `getPairwise()`. 
#'
#' @param result A matrix or data frame produced by `getPairwise()` where
#'   rownames are image IDs and colnames are concatenated marker pairs in the
#'   form `"from__to"`. Must contain at least one row with name matching `image`.
#' @param image Character scalar. Which image (row in `result`) to plot. If
#'   `NULL`, defaults to the first rowname of `result`.
#' @param colourGradient Character vector of length 3 giving the low, mid, and
#'   high colours for a diverging palette used by `scale_colour_gradient2()`.
#'   Default is `c("#4575B4", "white", "#D73027")`.
#' @param marksToPlot Optional character vector of marker names. If supplied, the
#'   plot is restricted to rows and columns where both `from` and `to` are in
#'   this set.
#' @param limits Numeric length-2 vector giving the lower and upper caps applied
#'   to association values for colour mapping. Use `NULL` to avoid
#'   clamping. Point sizes are clamped to `c(0, max(abs(limits)))`.
#'
#' @details
#' The function expects `colnames(result)` to contain `"__"` separating marker
#' names; otherwise an error is thrown. Values are transformed into two aesthetics:
#' \itemize{
#'   \item \code{value}: the (possibly clamped) signed association used for colour.
#'   \item \code{size}: the (possibly clamped) absolute association used for point size.
#' }
#' The colourbar shows three ticks at \code{lower}, \code{0}, and \code{upper}
#' with labels \code{"<lower"}, \code{"0"}, and \code{">upper"}.
#'
#' @return A \pkg{ggplot2} object.
#'
#' @examples
#' \dontrun{
#' # Minimal example (toy data)
#' set.seed(1)
#' mks <- c("A","B","C")
#' pairs <- as.vector(outer(mks, mks, paste, sep="__"))
#' res <- matrix(rnorm(length(pairs)*2), nrow = 2,
#'               dimnames = list(c("img1","img2"), pairs))
#'
#' p <- imageCrossPlot(res, image = "img1",
#'                        colourGradient = c("#4575B4","white","#D73027"),
#'                        marksToPlot = c("A","B","C"),
#'                        limits = c(-3, 3))
#' print(p)
#' }
#'
#' @importFrom ggplot2 ggplot aes geom_point scale_colour_gradient2 labs theme_classic guide_axis guides
#' @importFrom tidyr separate
#' @importFrom dplyr arrange filter
#' @importFrom scales squish
#' @seealso \code{\link{getPairwise}}
#' @export
imageCrossPlot = function(result,
                          image = NULL,
                          colourGradient = c("#4575B4", "white", "#D73027"),
                          marksToPlot = NULL,
                          limits = NULL){
  
  
  if(!any(grep("__", colnames(result)))) {
    stop("It doesn't look like this is output from `getPairwise()`")
  }
  
  if(is.null(image))image = rownames(result)[1]
  
  if(!image%in%rownames(result)) stop(paste(image, "is not in rownames of `result`"))
  
  data <- data.frame(test = colnames(result), 
                     value = as.numeric(result[image,]), 
                     size = abs(as.numeric(result[image,])))
  
  plotData = data |>
    tidyr::separate(test,
                    into = c("from", "to"),
                    sep = "__") |>
    dplyr::arrange(to, from)
  
  
  if(!is.null(marksToPlot)) {
    plotData = plotData |> 
      filter(to %in% marksToPlot) |> 
      filter(from %in% marksToPlot)
  }
  
  if(is.null(limits)) limits <- signif(range(data$value),2)
  
  val_limits  <- limits                   # e.g. c(-10, 10)
  size_limits <- c(0, max(abs(limits)))   # e.g. c(0, 10)
  
  # colour scale
  val_breaks <- c(val_limits[1], 0, val_limits[2])
  val_labels <- c(
    paste0("< ", val_limits[1]),
    "0",
    paste0("> ", val_limits[2])
  )
  
  # size scale
  size_breaks <- size_limits
  size_labels <- c(
    paste0("< ", size_limits[1]),
    paste0("> ", size_limits[2])
  )
  
  plotData$size_clamped <- pmin(pmax(plotData$size, size_limits[1]), size_limits[2])
  
  plot <- ggplot(plotData, aes(x = to, y = from)) +
    geom_point(aes(size = size_clamped, colour = value)) +
    scale_colour_gradient2(
      low = colourGradient[[1]], mid = colourGradient[[2]], high = colourGradient[[3]],
      midpoint = 0,
      limits = val_limits,
      oob = scales::squish,
      breaks = val_breaks,
      labels = val_labels,
      name = "Spatial association"
    ) +
    guides(size = "none") + 
    scale_x_discrete(guide = guide_axis(angle = 45)) +
    labs(x = NULL, y = NULL) +
    theme_classic()
  
  return(plot)
  
}

#' Plots an image with specified from and to cell types.
#' 
#' @param cells A SummarizedExperiment object.
#' @param imageToPlot The ID of the image to be plotted.
#' @param from The "from" cell type.
#' @param to The "to" cell type.
#' @param imageID The name of the imageID column in the SummarizedExperiment object.
#' @param cellType The name of the cellType column in the SummarizedExperiment object. 
#' @param spatialCoords The names of the spatialCoords column if using a SingleCellExperiment.
#' 
#' @return A ggplot object.
#' 
#' @examples
#' data("diabetesData")
#' plotImage(diabetesData, "A09", from = "acinar", to = "alpha")
#' 
#' @export
#' @import dplyr 
#' @import ggplot2 
#' @import SummarizedExperiment
#' @import SpatialExperiment
plotImage = function(cells, 
                     imageToPlot, 
                     from,
                     to,
                     imageID = "imageID", 
                     cellType = "cellType",
                     spatialCoords = c("x", "y")) {
  
  if (!is(cells, "SummarizedExperiment")) {
    stop(paste("Please provide a SummarizedExperiment object as input."))
  }
  
  if (!imageID %in% colnames(colData(cells))) {
    stop(paste0(imageID, " not found in colData."))
  }
  
  if (!imageToPlot %in% unique(cells[[imageID]])) {
    stop(paste0("imageToPlot not found in ", imageID, " column."))
  }
  
  if (!cellType %in% colnames(colData(cells))) {
    stop(paste0(cellType, " not found in colData."))
  }
  
  if (class(cells) == "SingleCellExperiment") {
    if (!all(spatialCoords %in% colnames(colData(cells)))) {
      stop(paste0(spatialCoords, " not found in colData. "))
    }
  }
  
  if (length(spatialCoords) != 2) {
    stop(paste("Please provide x and y coordinates columns."))
  }
  
  if (!all(c(from, to) %in% unique(colData(cells)[[cellType]]))) {
    stop("from and/or to cell types not found in data.")
  }
  
  if (class(cells) == "SingleCellExperiment") {
    cells = SpatialExperiment(assays = assays(cells), 
                              colData = colData(cells), 
                              rowData = rowData(cells))
    spatialCoords(cells) = as.matrix(colData(cells)[, c(spatialCoords[1], spatialCoords[2])])
  }
  
  # filter for specific image
  subset = cells[, colData(cells)[[imageID]] == imageToPlot]
  
  coords = spatialCoords(subset) |> as.data.frame()
  cData = data.frame(x = coords[["x"]],
                     y = coords[["y"]],
                     cellType = subset[[cellType]])
  
  cData[[cellType]] = as.character(cData[[cellType]])
  cData = cData |> mutate(cellTypeNew = 
                            ifelse(cellType %in% c(from, to), cellType, "Other"))
  
  
  pal = setNames(c("#d6b11c", "#850f07"), c(from, to))
  
  ggplot() +
    stat_density_2d(data = cData, aes(x = x, y = y, fill = after_stat(density)), 
                    geom = "raster", 
                    contour = FALSE) +
    geom_point(data = cData |> filter(cellTypeNew != "Other"),
               aes(x = x, y = y, colour = cellTypeNew), size = 1) +
    scale_color_manual(values = pal) +
    scale_fill_distiller(palette = "Blues", direction = 1) +
    theme_classic() +
    labs(title = paste0(imageID, ": ", imageToPlot),
         color = cellType)
}

#' Plots result of signifPlot.
#'
#' @param results A spicy results object
#' @param type Whether to make a bubble plot or heatmap. Note: For survival results a bubble plot will be used. 
#' @param fdr TRUE if FDR correction is used.
#' @param breaks Vector of 3 numbers giving breaks used in legend. The first
#'     number is the minimum, the second is the maximum, the third is the
#'     number of breaks.
#' @param comparisonGroup A string specifying the name of the outcome group to compare with the base group.
#' @param colours Vector of colours to use to colour legend.
#' @param marksToPlot Vector of marks to include in plot.
#' @param cutoff significance threshold for circles in bubble plot.
#' @param contextColours Used for \code{\link[Statial]{Kontextual}} results. A named list specifying the colours for each context.
#'  By default the Tableau colour palette is used.
#' @param contextLabels Used for \code{\link[Statial]{Kontextual}} results. A named list to change the default labels for each context.
#'
#' @return a ggplot or pheatmap object
#'
#' @examples
#' data(spicyTest)
#'
#' p <- signifPlot(spicyTest, breaks = c(-3, 3, 0.5))
#' # plot includes unicode characters, do not use default pdf device
#' ggplot2::ggsave(p, filename = tempfile(), device = cairo_pdf)
#'
#' @export
#' @importFrom pheatmap pheatmap
#' @importFrom grDevices colorRampPalette
#' @importFrom stats p.adjust
#' @importFrom ggplot2
#'     ggplot scale_colour_gradient2 geom_point scale_shape_manual guides labs
#'     scale_color_manual theme_classic theme element_text aes guide_legend
#'     element_blank guide_colourbar
#' @importFrom ggforce geom_arc_bar geom_circle
#' @importFrom grDevices colors
#' @importFrom stats setNames
#' @import ggthemes 
#' @import ggh4x 
#' @importFrom grid grobTree polygonGrob gpar
#' @importFrom ggnewscale new_scale
signifPlot <- function(results,
                       fdr = FALSE,
                       type = "bubble",
                       breaks = NULL,
                       comparisonGroup = NULL,
                       colours = c("#4575B4", "white", "#D73027"),
                       marksToPlot = NULL,
                       cutoff = 0.05,
                       contextColours = NULL,
                       contextLabels = waiver()) {
  
  if (is.null(comparisonGroup)) {
    coef <- 2
  } else {
    coef <- which(levels(results$condition) == comparisonGroup)
  }
  marks <- unique(results$comparisons$from)
  
  if ("fromName" %in% names(results$comparisons)) {
    marks <- unique(c(
      results$comparisons$to,
      results$comparisons$from
    ))
  }
  
  
  if (is.null(marksToPlot)) marksToPlot <- marks
  
  
  if("survivalResults" %in% names(results)) {
    return(
      survBubble(result = results,
                 fdr = fdr,
                 cutoff = cutoff,
                 colourGradient = colours,
                 marksToPlot = marksToPlot,
                 contextColours = contextColours,
                 contextLabels = contextLabels)
    )
  }

  
  if (type == "bubble") {
    return(
      bubblePlot(
        results, fdr, breaks, coef,
        colours = colours, cutoff = cutoff, marksToPlot = marksToPlot,
        contextColours = contextColours, contextLabels = contextLabels
      )
    )
  }
  
  if (is.null(breaks)) breaks <- c(-3, 3, 0.5)
  breaks <- seq(from = breaks[1], to = breaks[2], by = breaks[3])
  pal <- grDevices::colorRampPalette(colours)(length(breaks))
  
  
  pVal <- results$p.value[, coef]
  
  if (min(pVal, na.rm = TRUE) == 0) {
    pVal[pVal == 0] <-
      pVal[pVal == 0] + 10^floor(log10(min(pVal[pVal > 0], na.rm = TRUE)))
  }
  
  if (fdr) {
    pVal <- stats::p.adjust(pVal, method = "fdr")
  }
  
  isGreater <- results$coefficient[, coef] > 0
  
  pVal <- log10(pVal)
  
  pVal[isGreater] <- abs(pVal[isGreater])
  
  pVal <- matrix(pVal, nrow = length(marks))
  colnames(pVal) <- marks
  rownames(pVal) <- marks
  
  
  
  
  heatmap <- pheatmap::pheatmap(
    pVal[marksToPlot, marksToPlot],
    color = pal,
    breaks = breaks,
    cluster_rows = FALSE,
    cluster_cols = FALSE
  )
  
  heatmap
}

# Function to create x and y points for a half circle (left or right) ---> CHANGE HERE
half_circle_coords = function(shape = "left", num_points = 100) {
  # Generate points from -pi/2 to pi/2 for vertical half circles
  t = seq(-pi / 2, pi / 2, length.out = num_points)
  if (shape == "left") {
    x = 0.5 - 0.5 * cos(t) # Shift to the left half
  } else {
    x = 0.5 + 0.5 * cos(t)  # Shift to the right half
  }
  y = 0.5 + 0.5 * sin(t) # Vertical component
  list(x = c(x,x[1])-mean(c(x,x[1]))+0.5, y = c(y,y[1]))
}

# Custom draw_key function to draw a left half circle in the legend using polygonGrob ----> CHANGE HERE
draw_key_half_circle = function(data, params, shape) {
  if(data$shape == 16){
    coords <- half_circle_coords(shape = "left")
    grid::grobTree(
      grid::polygonGrob(
        x = coords$x, y = coords$y,
        gp = grid::gpar(fill = "black", col = "black")
      )
    )
  } else{
    coords <- half_circle_coords(shape = "right")
    grid::grobTree(
      grid::polygonGrob(
        x = coords$x, y = coords$y,
        gp = grid::gpar(fill = "black", col = "black")
      )
    )
  }
}





# Function to create x and y points for a half circle (left or right)
half_circle_coords = function(shape = "left", num_points = 100) {
  # Generate points from -pi/2 to pi/2 for vertical half circles
  t = seq(-pi / 2, pi / 2, length.out = num_points)
  if (shape == "left") {
    x = 0.5 - 0.5 * cos(t) # Shift to the left half
  } else {
    x = 0.5 + 0.5 * cos(t)  # Shift to the right half
  }
  y = 0.5 + 0.5 * sin(t) # Vertical component
  list(x = c(x,x[1])-mean(c(x,x[1]))+0.5, y = c(y,y[1]))
}

# Custom draw_key function to draw a left half circle in the legend using polygonGrob 
draw_key_half_circle = function(data, params, shape) {
  if(data$shape == 16){
    coords <- half_circle_coords(shape = "left")
    grid::grobTree(
      grid::polygonGrob(
        x = coords$x, y = coords$y,
        gp = grid::gpar(fill = "black", col = "black")
      )
    )
  } else{
    coords <- half_circle_coords(shape = "right")
    grid::grobTree(
      grid::polygonGrob(
        x = coords$x, y = coords$y,
        gp = grid::gpar(fill = "black", col = "black")
      )
    )
  }
}




#' @import ggthemes
#' @import ggh4x
#' @importFrom ggnewscale new_scale
bubblePlot <- function(test,
                       fdr,
                       breaks,
                       coef,
                       colours = c("blue", "white", "red"),
                       cutoff = 0.05,
                       marksToPlot,
                       contextColours = NULL,
                       contextLabels = waiver()) {
  

  
  if (is.null(test$alternateResult)) {
    test$alternateResult <- FALSE
  }
  
  if (test$alternateResult ) {
    groupA <- test$coefficient[, 1]
    groupB <- (test$coefficient[, 1] + test$coefficient[, coef])
  } else {
    groupA <- test$coefficient[, 1] * sqrt(pi) * 2 / sqrt(10) / 100
    groupB <- (
      test$coefficient[, 1] + test$coefficient[, coef]
    ) * sqrt(pi) * 2 / sqrt(10) / 100

  }
  
  
  cellTypeA <- factor(test$comparisons$from)
  cellTypeB <- factor(test$comparisons$to)
  
  
  pvalue = test$p.value[, coef]
  sig <- pvalue < cutoff
  sigLab <- paste0("p-value < ", cutoff)
  
  
  if (fdr) {
    pvalue = p.adjust(test$p.value[, coef], "fdr")
    sig <- pvalue < cutoff
    sigLab <- paste0("fdr < ", cutoff)
  }
  
  size <- -log10(pvalue)
  
  
  df <- data.frame(
    cellTypeA, cellTypeB, groupA, groupB, size,
    stat = test$statistic[, coef], pvalue = pvalue,
    sig = factor(sig, levels= c("FALSE", "TRUE"))
  )
  rownames(df) <- rownames(test$statistic)
  
  
  if (isTRUE(test$isKontextual)) {
    df$parent = test$comparisons$parent
  }

  df <- df[df$cellTypeA %in% marksToPlot & df$cellTypeB %in% marksToPlot, ]
  
  df$cellTypeA <- droplevels(df$cellTypeA)
  df$cellTypeB <- droplevels(df$cellTypeB)
  
  df.shape <- data.frame(
    cellTypeA = c(NA, NA), cellTypeB = c(NA, NA), size = c(1, 1),
    condition = c(
      levels(test$condition)[1], levels(test$condition)[coef]
    )
  )
  
  
  
  if(is.null(breaks)) {
    groupAB <- c(groupA, groupB)
    
    limits <- c(min(groupAB, na.rm = TRUE), max(groupAB, na.rm = TRUE)) |>
      round(1)
    breaks <- seq(from = limits[1], to = limits[2], by = diff(limits) / 5)
    
  } else {
    limits <- c(breaks[1], breaks[2])
    breaks <- seq(from = breaks[1], to = breaks[2], by = breaks[3])
  }
  
  midpoint <- 0
  
  if(test$alternateResult && !isTRUE(test$isKontextual)){
    midpoint <- (breaks[1] + breaks[length(breaks)])/2
  }
  
  
  df$groupA <- pmax(pmin(df$groupA, limits[2]), limits[1])
  df$groupB <- pmax(pmin(df$groupB, limits[2]), limits[1])
  
  pal <- grDevices::colorRampPalette(colours)(length(breaks)) # nolint
  
  labels <- round(breaks, 3)
  labels[1] <- "avoidance"
  labels[length(labels)] <- "attraction"


  if(isTRUE(test$isKontextual)) {
    df = df |> 
      group_by(parent) |>
      mutate(
        cellTypeB_numeric = as.numeric(droplevels(cellTypeB)),
        cellTypeB_id = factor(paste(parent, cellTypeB, sep = "_"))
      ) |>
      ungroup()
    
  } else {
    
    df = df |> 
      mutate(
        cellTypeB_numeric = as.numeric(cellTypeB),
        cellTypeB_id = cellTypeB
      )
  }
  
  plot = ggplot2::ggplot(df, ggplot2::aes(x = cellTypeB_id, y = cellTypeA)) +
    ggplot2::scale_fill_gradient2(
      low = colours[1], mid = colours[2], high = colours[3],
      midpoint = midpoint, breaks = breaks, labels = labels, limits = limits
    ) +
    ggplot2::geom_point(ggplot2::aes(col = sigLab), size = -1) +
    ggplot2::geom_point(ggplot2::aes(size = size), x = 100000, y = 10000000) +
    ggforce::geom_arc_bar(
      ggplot2::aes(
        fill = groupB, r = pmax(size / max(size, na.rm = TRUE) / 2, 0.15),
        r0 = 0, x0 = cellTypeB_numeric, y0 = as.numeric(cellTypeA),
        start = 0, end = pi, x = NULL, y = NULL
      ),
      color = NA
    ) +
    ggforce::geom_arc_bar(
      ggplot2::aes(
        fill = groupA, r = pmax(size / max(size, na.rm = TRUE) / 2, 0.15),
        r0 = 0, x0 = cellTypeB_numeric, y0 = as.numeric(cellTypeA),
        start = pi, end = 2 * pi, x = NULL, y = NULL
      ),
      colour = NA
    ) +
    ggplot2::geom_point(
      data = df.shape, ggplot2::aes(shape = condition), x = 10000, y = 10000,
      key_glyph = draw_key_half_circle 
    ) +
    scale_x_discrete(breaks = df$cellTypeB_id, labels = df$cellTypeB,
                     guide = guide_axis(angle = 45)) +
    ggplot2::theme_classic() +
    ggplot2::labs(
      x = "Cell type j", y = "Cell type i", size = "-log10 p-value",
      colour = NULL, fill = "Localisation", shape = "Condition"
    ) +
    ggplot2::guides(
      shape = ggplot2::guide_legend(order = 3,override.aes = list(size = 5)),
      #fill = ggplot2::guide_colourbar(order = 4), # adding this line breaks the ggnewscales package
      size = ggplot2::guide_legend(order = 2),
      colour = ggplot2::guide_legend(
        order = 1, override.aes = list(size = 5, shape = 1, col = "black")
      )
    )
  
  
  # Plots black circle outlines, only if there are significant results.
  if(nrow(df[df$sig == "TRUE", ]) > 0) {
    plot = plot + 
      ggforce::geom_circle(
        data = df[df$sig == "TRUE", ], ggplot2::aes(
          r = pmax(size / max(size, na.rm = TRUE) / 2, 0.15),
          x0 = cellTypeB_numeric, y0 = as.numeric(cellTypeA),
          x = NULL, y = NULL
        ), colour = "black"
      ) 
  }
  
  # Adds context panels if using Kontextual results.
  if(isTRUE(test$isKontextual)) {
    if(is.null(contextColours)) {
      # Defining colour palette for Context
      palette = ggthemes::ggthemes_data$tableau$`color-palettes`$regular$`Tableau 10`$value
      
      if (length(unique(df$parent)) > 10) {
        palette = ggthemes::ggthemes_data$tableau$`color-palettes`$regular$`Tableau 20`$value
      }
      
      contextColours = palette[1:length(unique(df$parent))]
      names(contextColours) = levels(df$parent)
    }
    
    # Assigning contextColours to the strip
    strip = ggh4x::strip_themed(background_x = ggh4x::elem_list_rect(fill = contextColours))
    
    plot = plot +
      ggh4x::facet_grid2( ~ parent,
                          scales = "free",
                          space = "free",
                          strip = strip) +
      theme(
        strip.text = element_text(size = -1),
        strip.clip = "off",
        strip.background = element_rect(linewidth = 0),
        panel.spacing = unit(0.4, 'lines')
      ) +
      ggnewscale::new_scale("fill") +
      geom_tile(aes(fill = parent), alpha = -1) +
      scale_fill_manual(values = contextColours, labels = contextLabels) +
      labs(fill = "Context") +
      ggplot2::guides(fill = guide_legend(order = 6, override.aes = list(alpha = 1)))
    
  }
  
  return(plot)
  
}

#' Plots survival results from spicy.
#'
#' @param result A spicyResults object that contains survival results.
#' @param fdr TRUE if FDR correction is used.
#' @param cutoff Significance threshold for circles in bubble plot.
#' @param colourGradient A vector of colours, used to define the low, medium, and high values for the colour scale.
#' @param marksToPlot Vector of marks to include in bubble plot.
#' @param contextColours Used for \code{\link[Statial]{Kontextual}} results. A named list specifying the colours for each context.
#'  By default the Tableau colour palette is used.
#' @param contextLabels Used for \code{\link[Statial]{Kontextual}} results. A named list to change the default labels for each context.
#'
#' 
#' @return A ggplot object.
#' 
#' @import ggh4x 
#' @import ggplot2
#' @import ggthemes
survBubble = function(result,
                      fdr = FALSE,
                      cutoff = 0.05,
                      colourGradient = c("#4575B4", "white", "#D73027"),
                      marksToPlot = NULL,
                      contextColours = NULL,
                      contextLabels = waiver()){
  
  
  if(!"survivalResults" %in% names(result)) {
    stop("Survival results are missing, please run spicy with survival outcomes.")
  }
  
  survivalResults = result$survivalResults

  
  if(isTRUE(result$isKontextual)) {
    plotData = survivalResults |>
      tidyr::separate(test,
               into = c("from", "to", "parent"),
               sep = "__") |>
      dplyr::arrange(parent, to, from) |>
      dplyr::mutate(toParent = paste(to, parent, sep = "__"))
    
    
    
    if(is.null(contextColours)) {
      # Defining colour palette for Context
      palette = ggthemes::ggthemes_data$tableau$`color-palettes`$regular$`Tableau 10`$value
      
      if (length(unique(plotData$parent)) > 10) {
        palette = ggthemes::ggthemes_data$tableau$`color-palettes`$regular$`Tableau 20`$value
      }
      
      contextColours = palette[1:length(unique(plotData$parent))]
      names(contextColours) = levels(plotData$parent)
    }
    
    # Assigning contextColours to the strip
    strip = ggh4x::strip_themed(background_x = ggh4x::elem_list_rect(fill = contextColours)
    )
    
  } else {
    plotData = survivalResults |>
      tidyr::separate(test, into = c("from", "to"), sep = "__") |>
      dplyr::arrange(to, from)
  }
  
  if(!is.null(marksToPlot)) {
    plotData = plotData |> 
      filter(to %in% marksToPlot) |> 
      filter(from %in% marksToPlot)
  }
  
  sigLab <- paste0("p-value < ", cutoff)
  
  if(fdr){
    plotData$p.value = p.adjust(plotData$p.value, "fdr")
    sigLab <- paste0("fdr < ", cutoff)
  }
  
  plotData = plotData |>
    mutate(
      sig = p.value < cutoff,
      logP = -log10(p.value),
      size = logP / max(logP, na.rm = TRUE),
      from = factor(from),
      to = factor(to)
    )
  
  plot = ggplot(plotData, aes(x = to, y = from)) +
    ggplot2::geom_point(ggplot2::aes(size = pmax(logP/2, 0.15), colour = coef)) +  
    ggplot2::geom_point(data = dplyr::filter(plotData, sig == TRUE),
                        aes(size = pmax(logP/2, 0.15)),
                        shape = 21) +
    ggplot2::geom_point(ggplot2::aes(shape = sigLab), size = -1) + 
    scale_colour_gradient2(low = colourGradient[[1]],
                           mid = colourGradient[[2]],
                           high = colourGradient[[3]],
                           midpoint = 0) +
    scale_size(range = c(2, 6)) +
    scale_x_discrete(guide = guide_axis(angle = 45)) +
    labs(colour = "CoxPH \ncoefficient",
         shape = NULL,
         x = NULL,
         y = NULL) +
    ggplot2::guides(shape = ggplot2::guide_legend(order = 1, override.aes = list(size=5, shape = 1, col = "black")),
                    colour = ggplot2::guide_colourbar(order = 2),
                    size = "none") +
    theme_classic() 
  
  
  if(isTRUE(result$isKontextual)) {
    # Adding context information to the plot
    # plot = plot + 
    #   geom_tile(aes(fill = parent), alpha = -1) +
    #   ggh4x::facet_grid2(~parent, scales = "free", space = "free", strip = strip) +
    #   scale_fill_manual(values = contextColours,
    #                     labels = contextLabels) +
    #   labs(fill = "Context") +
    #   ggplot2::guides(fill = guide_legend(order = 2, override.aes = list(alpha = 1))) +
    #   theme(strip.text = element_text(size = -1),
    #         strip.clip = "off",
    #         strip.background = element_rect(linewidth = 0),
    #         panel.spacing = unit(0.4,'lines'))
    message("ggplot2 broke this functionality when it released v 4.0.0")
    NULL
  }
  
  
  return(plot)
  
}

#' Plots boxplot for a specified cell-cell relationship
#'
#' @param results Data frame obtained from spicy.
#' @param from Cell type which you would like to compare to the to cell type.
#' @param to Cell type which you would like to compare to the from cell type.
#' @param rank Ranking of cell type in terms of p-value, the smaller the p-value
#'   the higher the rank.
#'
#' @return a ggplot2 boxplot
#'
#' @examples
#' data(spicyTest)
#' 
#' spicyBoxPlot(spicyTest,
#'              rank = 1)
#'
#' @export
#' @importFrom ggplot2 ggplot scale_colour_gradient2 geom_point scale_shape_manual guides labs scale_color_manual theme_classic theme element_text aes guide_legend element_blank guide_colourbar

spicyBoxPlot <- function(results,
                         from = NULL,
                         to = NULL,
                         rank = NULL) {
  
  if(is.null(c(from, to, rank))) {
    stop("Please specify either a pairwise relationship or rank")
  }
  
  pVal <- results$p.value
  
  if(is.null(rank)) {
    if(length(c(from, to)) == 1) {
      stop("Please specify both from and to parameters")
    }
    pairName <- paste0(from, "__", to)
  }
  
  if(!is.null(rank)) {
    pVal <- pVal[order(pVal[, 2]),]
    
    pairName <- rownames(pVal)[rank]
    from <- unlist(strsplit(pairName, split = "__"))[1]
    to <- unlist(strsplit(pairName, split = "__"))[2]
  }
  
  df <- data.frame(imageID = results$imageID, 
                   pairwiseAssoc = results$pairwiseAssoc[[pairName]],
                   condition = results$condition)
  
  if(results$alternateResult) {
    ylabel <- "Alternate Result"
  } else {
    ylabel <- "L Function"
  }
  
  ggplot2::ggplot(df, ggplot2::aes(x = condition, y = pairwiseAssoc, fill = condition)) +
    ggplot2::geom_boxplot() +
    # ggplot2::geom_dotplot(binaxis = "y", stackdir = "center", dotsize = 0.5) +
    ggplot2::ggtitle(paste0("L-function values between ", from, " cells and ", to, " cells")) +
    ggplot2::xlab("Condition") + 
    ggplot2::ylab(ylabel) +
    ggplot2::theme_classic()
}

# check if alternativeResults is kontextual
isKontextual <- function(kontextualResult) {
  
    return("kontexutal"%in% names(kontextualResult))
}

#' A format SummarizedExperiment and data.frame objects into a canonical form.
#'
#' @importFrom SummarizedExperiment colData
#' @importFrom magrittr %>%
#' @importFrom SpatialExperiment spatialCoords
#' @importFrom methods is
#' @importFrom dplyr select any_of rename mutate group_by ungroup
#' @importFrom cli cli_abort cli_inform
#' @noRd
.format_data <- function(
    cells, imageIDCol, cellTypeCol, spatialCoordCols, verbose = FALSE) {
    if (is(cells, "data.frame")) {
        # pass
    } else if (is(cells, "SpatialExperiment")) {
        spatialCoords <- names(spatialCoords(cells))
        cells <- cells %>%
            colData() %>%
            data.frame() %>%
            dplyr::select(-dplyr::any_of(c("x", "y"))) %>%
            cbind(spatialCoords(cells) %>% data.frame())
    } else if (is(cells, "SingleCellExperiment")) {
        cells <- cells %>%
            colData() %>%
            data.frame()
    } else {
        temp <- tryCatch(
            expr = {
                as.data.frame(cells)
            }, error = NULL
        )
        if (is.null(temp)) {
            cli::cli_abort(
                c(
                    "x" = "{.var cells} is an unsupported class: {.cls {class(cells)}}. \n", # nolint
                    "i" = "data.frame (or coercible), SingleCellExperiment and SpatialExperiment are currently supported." # nolint
                )
            )
        }
        cells <- temp
    }

    for (col in c(
        imageIDCol = imageIDCol,
        cellTypeCol = cellTypeCol,
        spatialCoordCols_x = spatialCoordCols[1],
        spatialCoordCols_y = spatialCoordCols[2]
    )) {
        if (!col %in% colnames(cells)) {
            cli::cli_abort(c(
                "x" = "Specified {.var {names(col)}} ({.emph {col}}) is not in {.var cells}.", # nolint
                "i" = "{.code names(cells)}: {names(cells)}"
            ))
        }
    }
  

    needed <- cells %>%
      select(!!imageIDCol, !!cellTypeCol, spatialCoordCols[1], spatialCoordCols[2]) %>%
      dplyr::rename(
        imageID = !!imageIDCol,
        cellType = !!cellTypeCol,
        x = spatialCoordCols[1],
        y = spatialCoordCols[2]
      )
  
    cells <- cells %>%
      dplyr::select(-dplyr::any_of(c("imageID", "cellType", "x", "y"))) %>%
        cbind(needed)
    
    # Check if imageID is a factor, if not make it as factor whilst keeping order
    if(class(cells$imageID) != "factor") {
      cells <- cells %>%
        dplyr::mutate(imageID = factor(imageID, levels = unique(imageID)))
    }
    
    # Check if imageID is a factor, if not make it as factor whilst keeping order
    if(class(cells$cellType) != "factor") {
      cells <- cells %>%
        dplyr::mutate(cellType = factor(cellType, levels = unique(cellType)))
    }
    
    # Order the cells by imageID. This is important for when the data is split downstream
    cells <- cells[order(cells$imageID),]  
    
    
    # create cellID if it does not exist
    if (is.null(cells$cellID)) {
        if (verbose) {
            cli::cli_inform(
                "No column called cellID. Creating one."
            )
        }
        cells <- dplyr::mutate(
            cells,
            cellID = paste0("cell", "_", dplyr::row_number())
        )
    }

    # create imageCellID if it does not exist
    if (is.null(cells$imageCellID)) {
        if (verbose) {
            cli::cli_inform(
                "No column called imageCellID. Creating one."
            )
        }
        cells <- cells %>%
            dplyr::group_by(imageID) %>%
            dplyr::mutate(
                imageCellID = paste0(imageID, "_", dplyr::row_number())
            ) %>%
            dplyr::ungroup()
    }
    
    cells
}

getCellSummary <- function(
    data,
    imageID = NULL,
    bind = TRUE) {
    data %>%
        dplyr::filter(
            if (!is.null(!!imageID)) imageID == !!imageID else TRUE
        ) %>%
        dplyr::select(imageID, cellID, imageCellID, x, y, cellType) %>%
        # dplyr::mutate(imageID = factor(imageID, levels = unique(imageID))) %>%
        S4Vectors::DataFrame() %>%
        {
            if (bind) . else S4Vectors::split(., .$imageID)
        }
}


getImageID <- function(x, imageID = NULL) {
    x %>%
        dplyr::filter(
            if (!is.null(!!imageID)) imageID == !!imageID else TRUE
        ) %>%
        dplyr::pull(imageID)
}

getCellType <- function(x, imageID = NULL) {
    x %>%
        dplyr::filter(
            if (!is.null(!!imageID)) imageID == !!imageID else TRUE
        ) %>%
        dplyr::pull(cellType)
}

getImagePheno <- function(x,
                          imageID = NULL,
                          bind = TRUE,
                          expand = FALSE) {
    x <- x[!duplicated(x$imageID),]
    x <- x %>%
      dplyr::mutate(imageID = factor(imageID, levels = unique(imageID)))
    # x <- x %>%
    #     dplyr::filter(
    #         if (!is.null(!!imageID)) imageID == !!imageID else TRUE
    #     ) %>%
    #     dplyr::select(-cellID, -imageCellID, -x, -y, -cellType) %>%
    #     dplyr::mutate(imageID = as.factor(imageID)) %>%
    #     {
    #         if (expand) . else dplyr::distinct(.)
    #     } %>%
    #     S4Vectors::DataFrame()
    # if (expand) rownames(x) <- x$imageID
    x
}

#' A function to handle validity of argumemts/check for deprecated arguments
#'
#' @importFrom lifecycle deprecate_warn
#' @importFrom methods is
#' @noRd
argumentChecks = function(function_name, user_vals) {
  
  rlang::local_options(lifecycle_verbosity = "warning")  
  
  # handle deprecated arguments
  handle_deprecated = function(old_arg, new_arg, user_vals) {
    if (old_arg %in% names(user_vals) && !(new_arg %in% names(user_vals))) {
      deprecate_warn("1.18.0", paste0(function_name, "(", old_arg, ")"), paste0(function_name, "(", new_arg, ")"))
      assign(new_arg, user_vals[[old_arg]], envir = sys.frame(sys.parent(1)))
    }
  }
  
  handle_deprecated("imageIDCol", "imageID", user_vals)
  handle_deprecated("cellTypeCol", "cellType", user_vals)
  handle_deprecated("spatialCoordCols", "spatialCoords", user_vals)
  handle_deprecated("nCores", "cores", user_vals)
  handle_deprecated("BPPARAM", "cores", user_vals)
  handle_deprecated("Rs", "r", user_vals)
  
  # enforce mutually exclusive arguments
  check_exclusive = function(arg_set, user_vals) {
    provided_args = intersect(arg_set, names(user_vals)) 
    if (length(provided_args) > 1) {
      stop(paste("Please specify only one of", paste(shQuote(arg_set), collapse = ", "), "\n"))
    }
  }
  
  check_exclusive(c("cellTypeCol", "cellType"), user_vals)
  check_exclusive(c("imageIDCol", "imageID"), user_vals)
  check_exclusive(c("spatialCoordCols", "spatialCoords"), user_vals)
  check_exclusive(c("cores", "nCores", "BPPARAM"), user_vals)
  check_exclusive(c("Rs", "r"), user_vals)

  # validity checks for cores/nCores/BPPARAM
  if ("nCores" %in% names(user_vals)) {
    deprecate_warn("1.18.0", paste0(function_name, "(nCores)"), paste0(function_name, "(cores)"))
    
    if (is(user_vals$nCores, "numeric")) {
      assign("cores", simpleSeg:::generateBPParam(cores = user_vals$nCores), envir = parent.frame())
    } else if (is(user_vals$nCores, "MulticoreParam") || is(user_vals$nCores, "SerialParam")) {
      assign("cores", user_vals$nCores, envir = sys.frame(sys.parent(1)))
    } else {
      stop("'nCores' must be either a numeric value, or a MulticoreParam or SerialParam object.\n")
    }
    
  } else if ("BPPARAM" %in% names(user_vals)) {
    deprecate_warn("1.18.0", paste0(function_name, "(BPPARAM)"), paste0(function_name, "(cores)"))
    
    if (is(user_vals$BPPARAM, "MulticoreParam") || is(user_vals$BPPARAM, "SerialParam")) {
      assign("cores", user_vals$BPPARAM, envir = sys.frame(sys.parent(1)))
    } else {
      stop("'BBPARAM' must be a MulticoreParam or SerialParam object.")
    }
    
  } else if ("cores" %in% names(user_vals)) {
    if (is(user_vals$cores, "numeric")) {
      assign("cores", simpleSeg:::generateBPParam(cores = user_vals$cores))
    } else if (is(user_vals$cores, "MulticoreParam") || is(user_vals$cores, "SerialParam")) {
      assign("cores", user_vals$cores, sys.frame(sys.parent(1)))
    } else {
      stop("'cores'  must be either a numeric value, or a MulticoreParam or SerialParam object.\n")
    }
  }
}

test_that("Data formatting works of different formats.", {
  diabetesData_SPE <- SpatialExperiment::SpatialExperiment(
    diabetesData,
    colData = SummarizedExperiment::colData(diabetesData)
  )

  SpatialExperiment::spatialCoords(diabetesData_SPE) <- data.frame(
    SummarizedExperiment::colData(diabetesData_SPE)$x,
    SummarizedExperiment::colData(diabetesData_SPE)$y
  ) |> as.matrix()
  SpatialExperiment::spatialCoordsNames(diabetesData_SPE) <- c("x", "y")

  expect_no_error(
    .format_data(diabetesData_SPE, "imageID", "cellType", c("x", "y"), FALSE)
  )
  expect_no_error(
    .format_data(
      SummarizedExperiment::colData(diabetesData),
      "imageID", "cellType", c("x", "y"), FALSE
    )
  )
  expect_no_error(
    .format_data(
      SummarizedExperiment::colData(diabetesData) %>%
        as.data.frame() %>%
        select(-cellID, -imageCellID),
      "imageID", "cellType", c("x", "y"), FALSE
    )
  )
})

test_that(
  "Test .format_data input validation",
  {
    expect_error(
      .format_data(
        diabetesData, "imageIDs", "cellType", c("x", "y"), FALSE
      )
    )
    expect_error(
      .format_data(
        diabetesData, "imageID", "cellType", c("x", "b"), FALSE
      )
    )
  }
)

#' Performs spatial tests on spatial cytometry data.
#'
#' @param cells A SummarizedExperiment or data frame that contains at least the  variables
#'   x and y, giving the location coordinates of each cell, and cellType.
#' @param condition A character specifying which column which contains the condition or `Surv` objects.
#' @param subject Vector of subject IDs corresponding to each image if cells is
#'   a data frame.
#' @param covariates Vector of covariate names that should be included in the
#'   mixed effects model as fixed effects.
#' @param imageID The name of the imageID column if using a SingleCellExperiment or SpatialExperiment.
#' @param cellType The name of the cellType column if using a SingleCellExperiment or SpatialExperiment.
#' @param spatialCoords The names of the spatialCoords column if using a SingleCellExperiment.
#' @param r A vector of the radii that the measures of association should be calculated over.
#' @param sigma A numeric variable used for scaling when fitting inhomogenous L-curves.
#' @param from vector of cell types which you would like to compare to the to vector.
#' @param to vector of cell types which you would like to compare to the from vector.
#' @param alternateResult A pairwise association statistic between each combination of celltypes in
#'   each image.
#' @param cores Number of cores to use for parallel processing or a BiocParallel MulticoreParam or SerialParam object.
#' @param minLambda Minimum value density for scaling when fitting inhomogeneous L-curves.
#' @param weights logical indicating whether to include weights based on cell counts.
#' @param weightsByPair logical indicating whether weights should be calculated for each cell type
#'   pair.
#' @param weightFactor numeric that controls the convexity of the weight function.
#' @param window 	Should the window around the regions be 'square', 'convex' or 'concave'.
#' @param window.length A tuning parameter for controlling the level of concavity when estimating concave windows.
#' @param edgeCorrect A logical indicating whether to perform edge correction.
#' @param includeZeroCells 	A logical indicating whether to include cells with zero counts in the pairwise association calculation.
#' @param verbose logical indicating whether to output messages.
#' @param BPPARAM \{DEPRECATED\} A BiocParallel MulticoreParam or SerialParam object. 
#' @param imageIDCol \{DEPRECATED\} The name of the imageID column if using a SingleCellExperiment or SpatialExperiment.
#' @param cellTypeCol \{DEPRECATED\} The name of the cellType column if using a SingleCellExperiment or SpatialExperiment.
#' @param spatialCoordCols \{DEPRECATED\} The names of the spatialCoords column if using a SingleCellExperiment.
#' @param nCores \{DEPRECATED\} Number of cores to use for parallel processing or a BiocParallel MulticoreParam or SerialParam object.
#' @param Rs \{DEPRECATED\} A vector of the radii that the measures of association should be calculated over.
#' @param ... Other options
#' @return Data frame of p-values.
#' @export
#'
#' @examples
#' data("diabetesData")
#'
#' # Test with random effect for patient on a pairwise combination of cell
#' # types.
#' spicy(diabetesData,
#'   condition = "stage", subject = "case",
#'   from = "Tc", to = "Th"
#' )
#'
#' # Test all pairwise combinations of cell types without random effect of
#' # patient.
#' \dontrun{
#' spicyTest <- spicy(diabetesData, condition = "stage", subject = "case")
#' }
#'
#' # Test all pairwise combination of cell types with random effect of patient.
#' \dontrun{
#' spicy(diabetesData, condition = "condition", subject = "subject")
#' }
#'
#' @aliases
#' spicy
#' spicy,spicy-method
#' @importFrom scam scam
#' @importFrom rlang .data
#' @importFrom tibble column_to_rownames
#' @importFrom lifecycle deprecate_soft
spicy <- function(cells,
                  condition,
                  subject = NULL,
                  covariates = NULL,
                  imageID = "imageID",
                  cellType = "cellType",
                  spatialCoords = c("x", "y"),
                  r = NULL,
                  sigma = NULL,
                  from = NULL,
                  to = NULL,
                  alternateResult = NULL,
                  cores = 1,
                  minLambda = 0.05,
                  weights = TRUE,
                  weightsByPair = FALSE,
                  weightFactor = 1,
                  window = "convex",
                  window.length = NULL,
                  edgeCorrect = TRUE,
                  includeZeroCells = FALSE,
                  verbose = FALSE,
                  BPPARAM = NULL,
                  imageIDCol = imageID,
                  cellTypeCol = cellType,
                  spatialCoordCols = spatialCoords,
                  nCores = cores,
                  Rs = r,
                  ...) {
  
  user_args = as.list(match.call())[-1]
  user_vals = lapply(user_args, eval, envir = parent.frame())
  argumentChecks("spicy", user_vals)
  
  if (is.null(BPPARAM)) {
    if (cores > 1 && .Platform$OS.type != "windows") {
      BPPARAM = BiocParallel::MulticoreParam(workers = cores)
    } else if (cores > 1) {
      BPPARAM = BiocParallel::SnowParam(workers = cores)
    } else {
      BPPARAM = BiocParallel::SerialParam()
    } 
  }
  
  if (is(cells, "SummarizedExperiment") || is(cells, "data.frame")) {
    cells <- .format_data(
      cells, imageID, cellType, spatialCoords, verbose
    )
  }

  if (is.null(from) || is.null(to)) {
    if (is.null(from)) {
      from <- as.character(unique(getCellType(cells)))
    }
    if (is.null(to)) {
      to <- as.character(unique(getCellType(cells)))
    }

    m1 <- rep(from, times = length(to))
    m2 <- rep(to, each = length(from))
    labels <- paste(m1, m2, sep = "__")
  } else {
    m1 <- from
    m2 <- to
    labels <- paste(m1, m2, sep = "__")
    if (any(duplicated(labels))) stop("There are duplicated from-to pairs")
  }


  if (any((!to %in% getCellType(cells)) | (!from %in% getCellType(cells)))) {
    stop("to and from need to be cell type in your data")
  }

  nCells <- table(getImageID(cells), getCellType(cells))
  
  
  if (!is.null(condition)) {
    conditionVector <- as.data.frame(getImagePheno(cells))[condition][, 1]
    
    if (!inherits(conditionVector, "Surv")) {
      wasFactor <- is.factor(conditionVector)
      
      if (!wasFactor) {
        conditionVector <- as.factor(conditionVector)
      }
      
      conditionVector <- droplevels(conditionVector)
      conditionVector <- relevel(conditionVector, ref = levels(conditionVector)[1])
      
      if (!wasFactor || TRUE) {  
        cli::cli_inform(
          paste0(
            if (!wasFactor) "Coercing condition into factor. " else "",
            "Dropping unused levels. Using ",
            condition, " = ", levels(conditionVector)[1],
            " as base comparison group. If this is not the desired base group,",
            " please convert cells$", condition, " into a factor and change the order of levels(cells$",
            condition, ") so that the base group is at index 1."
          )
        )
      }
    }
  }
  
  

  ## Check whether the subject parameter has a one-to-one mapping with image
  if (!is.null(subject)) {
    if (nrow(as.data.frame(unique(cells[, subject]))) == nrow(as.data.frame(unique(cells[, imageID])))) {
      subject <- NULL
      
      if(inherits(conditionVector, "Surv")) {
        warning("Your specified subject parameter has a one-to-one mapping with imageID. Converting to a coxph model instead of cox mixed effects model.")
      } else{
        warning("Your specified subject parameter has a one-to-one mapping with imageID. Converting to a linear model instead of mixed model.")
      }
      
    }  
  }

  
  spicyResult = list()

  
  comparisons <- data.frame(from = m1,
                            to = m2,
                            labels = labels)

  ## Find pairwise associations

  if (is.null(alternateResult)) {
    pairwiseAssoc <- getPairwise(cells,
        r = r,
        sigma = sigma,
        from = from,
        to = to,
        minLambda = minLambda,
        window = window,
        window.length = window.length,
        edgeCorrect = edgeCorrect,
        includeZeroCells = includeZeroCells,
        BPPARAM = BPPARAM
    )
    pairwiseAssoc <- as.data.frame(pairwiseAssoc)
    pairwiseAssoc <- pairwiseAssoc[labels]
    
  }
  
  
  if (!is.null(alternateResult)) {
    pairwiseAssoc <- alternateResult
    
    # Checking if Kontextual result.
    if (isTRUE(attr(alternateResult, "kontextualResult"))) {
      pairwiseAssoc <- alternateResult
      
      labels <- names(pairwiseAssoc)
      
      comparisons <- data.frame(labels) |>
        tidyr::separate(
          col = labels,
          into = c("from", "to", "parent"),
          sep = "__"
        ) |>
        dplyr::mutate(labels = paste(.data$from, .data$to, .data$parent, sep = "__"))
      
      m1 <- comparisons$from
      m2 <- comparisons$to
      
      spicyResult$isKontextual = TRUE
    
    }
  }
  
  
  weightFunction <- getWeightFunction(
    pairwiseAssoc, nCells, m1, m2, BPPARAM, weights, weightsByPair, weightFactor
  )
  
  # Matrix needed for survival analysis
  pairwiseAssocMatrix = pairwiseAssoc
  
  pairwiseAssoc <- as.list(data.frame(pairwiseAssoc))
  names(pairwiseAssoc) <- labels
  

  
  ## Survival model
  if(inherits(conditionVector, "Surv")){
    
    survivalResult = spatialSurv(
      measurementMat = pairwiseAssocMatrix,
      condition = condition,
      pheno = as.data.frame(getImagePheno(cells)),
      covariates = covariates,
      subject = subject,
      weights = weightFunction,
      BPPARAM = BPPARAM
    )
    
    spicyResult$survivalOutcome = conditionVector
    spicyResult = append(spicyResult, list("survivalResults" = survivalResult))
    
  }


  ## Linear model
  if (!inherits(conditionVector, "Surv") && is.null(subject) && !is.null(condition)) {
    if (verbose) {
      cli::cli_inform("Testing for spatial differences across conditions")
    }

    MoreArgs2 <-
      list(
        cells = cells,
        condition = condition,
        covariates = covariates,
        cellCounts = table(getImageID(cells), getCellType(cells)),
        pheno = as.data.frame(getImagePheno(cells))
      )


    linearModels <- mapply(
      spatialLM,
      spatAssoc = pairwiseAssoc,
      from = m1,
      to = m2,
      weightFunction = weightFunction,
      MoreArgs = MoreArgs2,
      SIMPLIFY = FALSE
    )

    lmResult <- cleanLM(linearModels, BPPARAM = BPPARAM)
    spicyResult = append(spicyResult, lmResult)
  }


  ## Mixed effects model
  if (!inherits(conditionVector, "Surv") && (!is.null(subject)) && !is.null(condition)) {
    if (verbose) {
      cli::cli_inform(
        "Testing for spatial differences across conditions accounting for multiple images per subject" # nolint
      )
    }

    MoreArgs2 <-
      list(
        cells = cells,
        subject = subject,
        condition = condition,
        covariates = covariates,
        cellCounts = table(getImageID(cells), getCellType(cells)),
        pheno = as.data.frame(getImagePheno(cells))
      )


    mixed.lmer <- mapply(
      spatialMEM,
      spatAssoc = pairwiseAssoc,
      from = m1,
      to = m2,
      weightFunction = weightFunction,
      MoreArgs = MoreArgs2,
      SIMPLIFY = FALSE
    )


    mixed.lmer <- lapply(mixed.lmer, function(x) {
      if (is(x, "lmerModLmerTest")) {
        if (x@devcomp$cmp["REML"] == -Inf) {
          return(NA)
        }
      }
      x
    })

    melmResult <- cleanMEM(mixed.lmer, BPPARAM = BPPARAM)
    spicyResult = append(spicyResult, melmResult)
  }

  
  if(!is.null(condition)){
    spicyResult$condition <- conditionVector
  }

  if (!is.null(subject)) {
    spicyResult$subject <- as.data.frame(getImagePheno(cells))[subject][, 1]
  }
  
  spicyResult$pairwiseAssoc <- pairwiseAssoc
  spicyResult$comparisons <- comparisons
  
  spicyResult$weights <- weightFunction
  spicyResult$nCells <- nCells
  
  spicyResult$imageIDs <- as.data.frame(getImagePheno(cells))["imageID"][, 1]
  spicyResult$alternateResult <- ifelse(is.null(alternateResult), FALSE, TRUE)
  
  spicyResult <- methods::new("SpicyResults", spicyResult)
  spicyResult
}




#' @importFrom dplyr bind_rows
cleanLM <- function(linearModels, BPPARAM) {
  tLm <- lapply(linearModels, function(LM) {
    if (is(LM, "lm")) {
      coef <- as.data.frame(t(summary(LM)$coef))
      coef <-
        split(coef, c("coefficient", "se", "statistic", "p.value"))
    } else {
      n <- data.frame(NA)
      colnames(n) <- "(Intercept)"
      coef <- list(coefficient = n, se = n, statistic = n, p.value = n)
    }
    coef
  })


  df <- do.call("rbind", tLm)

  df <- suppressWarnings(apply(df, 2, function(x) {
    dplyr::bind_rows(x)
  }))

  df <- lapply(df, function(x) {
    rownames(x) <- names(linearModels)
    x
  })
  df
}


#' @importFrom dplyr bind_rows
cleanMEM <- function(mixed.lmer, BPPARAM) {
  tLmer <- lapply(mixed.lmer, function(lmer) {
    if (is(lmer, "lmerMod")) {
      coef <- as.data.frame(t(summary(lmer)$coef))
      coef <-
        split(
          coef,
          c("coefficient", "se", "df", "statistic", "p.value")
        )
    } else {
      n <- data.frame(NA)
      colnames(n) <- "(Intercept)"
      coef <- list(
        coefficient = n, df = n, p.value = n, se = n, statistic = n
      )
    }
    coef
  })

  df <- do.call("rbind", tLmer)

  df <- suppressWarnings(apply(df, 2, function(x) {
    dplyr::bind_rows(x)
  }))

  df <- lapply(df, function(x) {
    rownames(x) <- names(mixed.lmer)
    x
  })
  df
}

#' Get statistic from pairwise L curve of a single image.
#'
#' @param cells A SummarizedExperiment that contains at least the
#'     variables x and y, giving the location coordinates of each cell, and
#'     cellType.
#' @param imageID
#'     The name of the imageID column if using a SingleCellExperiment or SpatialExperiment.
#' @param cellType The name of the cellType column if using a SingleCellExperiment or SpatialExperiment.
#' @param spatialCoords The names of the spatialCoords column if using a SingleCellExperiment.
#' @param r A vector of the radii that the measures of association should be calculated over.
#' @param sigma A numeric variable used for scaling when fitting inhomogenous L-curves.
#' @param from The 'from' cellType for generating the L curve.
#' @param to The 'to' cellType for generating the L curve.
#' @param cores Number of cores to use for parallel processing or a BiocParallel MulticoreParam or SerialParam object.
#' @param minLambda Minimum value density for scaling when fitting inhomogeneous L-curves.
#' @param window Should the window around the regions be 'square', 'convex' or 'concave'.
#' @param window.length A tuning parameter for controlling the level of concavity when estimating concave windows.
#' @param edgeCorrect A logical indicating whether to perform edge correction.
#' @param includeZeroCells A logical indicating whether to include cells with zero counts in the pairwise association.
#' @param BPPARAM \{DEPRECATED\} A BiocParallel MulticoreParam or SerialParam object. 
#' @param imageIDCol \{DEPRECATED\} The name of the imageID column if using a SingleCellExperiment or SpatialExperiment.
#' @param cellTypeCol \{DEPRECATED\} The name of the cellType column if using a SingleCellExperiment or SpatialExperiment.
#' @param spatialCoordCols \{DEPRECATED\} The names of the spatialCoords column if using a SingleCellExperiment.
#' @param nCores \{DEPRECATED\} Number of cores to use for parallel processing or a BiocParallel MulticoreParam or SerialParam object.
#' @param Rs \{DEPRECATED\} A vector of the radii that the measures of association should be calculated over.
#' calculation.
#' @return Statistic from pairwise L-curve of a single image.
#' @examples
#' data("diabetesData")
#' # Subset by imageID for fast example
#' selected_cells <- diabetesData[
#'   , SummarizedExperiment::colData(diabetesData)$imageID == "A09"
#' ]
#' pairAssoc <- getPairwise(selected_cells)
#' @export
#' @importFrom BiocParallel bplapply
#' @importFrom BiocParallel MulticoreParam
getPairwise <- function(
    cells,
    imageID = "imageID",
    cellType = "cellType",
    spatialCoords = c("x", "y"),
    r = NULL,
    sigma = NULL,
    from = NULL,
    to = NULL,
    cores = 1, 
    minLambda = 0.05,
    window = "convex",
    window.length = NULL,
    edgeCorrect = TRUE,
    includeZeroCells = FALSE,
    BPPARAM = NULL,
    imageIDCol = imageID,
    cellTypeCol = cellType,
    spatialCoordCols = spatialCoords,
    nCores = cores,
    Rs = r
    ) {
    
    
    user_args = as.list(match.call())[-1]
      
    tryCatch({
      user_vals = lapply(user_args, eval, envir = parent.frame())
      argumentChecks("getPairwise", user_vals)
    }, error = function(e) {
    if (grepl("object 'cells' not found", e$message)) {
        message("Skipping argument checks as `getPairwise()` is being called within `spicy()`")
    } else {
      stop(e)
    }
  })

  
  if (is(cells, "SummarizedExperiment")) {
    cells <- .format_data(
      cells, imageID, cellType, spatialCoords, FALSE
    )
  }
    
  if (is.null(BPPARAM)) {
    if (cores > 1 && .Platform$OS.type != "windows") {
      BPPARAM = BiocParallel::MulticoreParam(workers = cores)
    } else if (cores > 1) {
      BPPARAM = BiocParallel::SnowParam(workers = cores)
    } else {
      BPPARAM = BiocParallel::SerialParam()
    } 
  }  
    
  cells2 <- getCellSummary(cells, bind = FALSE)


  if (is.null(from)) from <- levels(cells2$cellType)
  if (is.null(to)) to <- levels(cells2$cellType)

  pairwiseVals <- BiocParallel::bplapply(cells2,
    inhomLPair,
    Rs = Rs,
    sigma = sigma,
    window = window,
    window.length = window.length,
    minLambda = minLambda,
    from = from,
    to = to,
    edgeCorrect = edgeCorrect,
    includeZeroCells = includeZeroCells,
    BPPARAM = BPPARAM
  )
  return(do.call("rbind", pairwiseVals))

  unlist(pairwiseVals)
}




#' Get proportions from a SummarizedExperiment.
#'
#' @param cells A SingleCellExperiment, SpatialExperiment or data.frame.
#' @param feature The feature of interest
#' @param imageID The imageID's
#'
#' @return Proportions
#'
#'
#' @examples
#' data("diabetesData")
#' prop <- getProp(diabetesData)
#' @export
#' @importFrom SummarizedExperiment colData
getProp <- function(cells, feature = "cellType", imageID = "imageID") {
  if (is.data.frame(cells)) {
    df <- cells[, c(imageID, feature)]
  }

  if (is(cells, "SingleCellExperiment") || is(cells, "SpatialExperiment")) {
    df <- as.data.frame(
      SummarizedExperiment::colData(cells)
    )[, c(imageID, feature)]
  }


  tab <- table(df[, imageID], df[, feature])
  tab <- sweep(tab, 1, rowSums(tab), "/")
  as.data.frame.matrix(tab)
}


#' @importFrom stats p.adjust
.show_SpicyResults <- function(df) {
  pval <- as.data.frame(df$p.value)
  cond <- colnames(pval)[grep("condition", colnames(pval))]
  message(df$test)
  message("Number of cell type pairs: ", nrow(pval), "\n")
  message("Number of differentially localised cell type pairs: \n")
  if (nrow(pval) == 1) {
    print(sum(pval[cond] < 0.05, na.rm = TRUE))
  }
  if (nrow(pval) > 1) {
    print(colSums(apply(pval[cond], 2, p.adjust, "fdr") < 0.05, na.rm = TRUE))
  }
}
setMethod(
  "show", methods::signature(object = "SpicyResults"), function(object) {
    .show_SpicyResults(object)
  }
)

#' @importFrom lmerTest lmer
#' @importFrom stats predict weights
#' @importFrom methods is
spatialMEM <-
  function(spatAssoc,
           from,
           to,
           cells,
           subject,
           condition,
           covariates,
           weightFunction,
           cellCounts,
           pheno) {
    spatAssoc[is.na(spatAssoc)] <- 0

    # count1 <- cellCounts[, from]
    # count2 <- cellCounts[, to]

    spatialData <-
      data.frame(
        "spatAssoc" = spatAssoc,
        "condition" = pheno[, condition],
        "subject" = pheno[, subject],
        "covariates" = pheno[covariates]
      )

    names(spatialData)[grep("covariates", names(spatialData))] <- covariates

    formula <- "spatAssoc ~ condition + (1|subject)"

    if (!is.null(covariates)) {
      formula <-
        paste("spatAssoc ~ condition + (1|subject)",
          paste(covariates, collapse = "+"),
          sep = "+"
        )
    }

    spatialData$weights <- weightFunction

    mixed.lmer <- suppressWarnings(suppressMessages(tryCatch(
      {
        lmerTest::lmer(stats::formula(formula),
          data = spatialData,
          weights = spatialData$weights # TODO: weights does not exist or is a funciton.
        )
      },
      error = function(e) {

      }
    )))
    if (!is(mixed.lmer, "lmerMod")) {
      return(NA)
    }


    mixed.lmer
  }

#' @importFrom stats predict lm
spatialLM <-
  function(spatAssoc,
           from,
           to,
           cells,
           condition,
           covariates,
           weightFunction,
           cellCounts,
           pheno) {
    count1 <- cellCounts[, from]
    count2 <- cellCounts[, to]


    spatialData <-
      data.frame(
        "spatAssoc" = spatAssoc,
        "condition" = pheno[, condition],
        "covariates" = pheno[, covariates]
      )

    names(spatialData)[grep("covariates", names(spatialData))] <- covariates

    formula <- "spatAssoc ~ condition"

    if (!is.null(covariates)) {
      formula <-
        paste("spatAssoc ~ condition",
          paste(covariates, collapse = "+"),
          sep = "+"
        )
    }

    lm1 <- tryCatch(
      {
        stats::lm(stats::formula(formula),
          data = spatialData,
          weights = weightFunction # TODO: check that this works correctly
        )
      },
      error = function(e) {

      }
    )

    if (!is(lm1, "lm")) {
      return(NA)
    }


    lm1
  }

#' @importFrom survival coxph
#' @importFrom dplyr bind_rows mutate rename select arrange across where 
#' @importFrom coxme coxme
#' @importFrom BiocParallel bplapply SerialParam
spatialSurv <- function(measurementMat,
                        condition,
                        pheno,
                        covariates = NULL,
                        subject = NULL,
                        weights = NULL,
                        remove = NULL,
                        BPPARAM = NULL) {
  
  result <- bplapply(colnames(measurementMat), function(test) {
    measurementCol <- measurementMat[, test]
    ind <- !(measurementCol %in% remove)
    
    spatialData = data.frame("measurementCol" = measurementCol, "surv" = pheno[, condition])
    
    formula = "surv ~ measurementCol"
    
    if (!is.null(subject)) {
      spatialData = data.frame(spatialData, "subject" = pheno[, subject])
      
      formula <- paste(formula, "(1 | subject)", sep = "+")
    }
    
    if (!is.null(covariates)) {
      spatialData = data.frame(spatialData, "covariates" = pheno[, covariates])
      names(spatialData)[grep("covariates", names(spatialData))] <- covariates
      
      formula <- paste(formula, paste(covariates, collapse = "+"), sep = "+")
    }
    
    formula = stats::formula(formula)
    
    if(is.null(weights)) {
      spatialData$weights = 1
    } else {
      spatialData$weights = weights[[test]]
    }
    
    if(is.null(subject)) {
    # If no random effects must use coxph
    fit <- survival::coxph(formula, data = spatialData, weights = spatialData$weights, subset = ind)
    result <- summary(fit)$coefficients["measurementCol", c("coef", "se(coef)", "Pr(>|z|)")]
  } else{
    # Drop factors for any factor columns in spatialData
    spatialData = spatialData %>%
      mutate(across(where(is.factor), ~ droplevels(.)))
    
    # Mixed effects survival
    fit <- coxme::coxme(formula, data = spatialData, weights = spatialData$weights, subset = ind)
    
    # from print.coxme() function
    beta <- unname(fit$coefficients)
    nvar <- length(beta)
    nfrail <- nrow(fit$var) - nvar
    
    se <- sqrt(diag(as.matrix(fit$var))[nfrail + 1:nvar])
    p_val <- 1 - pchisq((beta / se) ^ 2, 1)
    
    # Need to select first indexes of each value incase of multiple betas
    result <- c("coef" = beta[1], "se(coef)" = se[1], "Pr(>|z|)" = p_val[1])
  }
  
  return(result)
  }, BPPARAM = BPPARAM)

result <- result |>
  dplyr::bind_rows() |>
  dplyr::mutate(test = colnames(measurementMat)) |>
  dplyr::rename("se.coef" = "se(coef)", "p.value" = "Pr(>|z|)") |>
  dplyr::select(test, coef, se.coef, p.value) |>
  dplyr::arrange(p.value)

return(result)
}


###########################
#
#  Generate distances
#
###########################


#' @importFrom spatstat.geom owin convexhull ppp
#' @importFrom concaveman concaveman
makeWindow <-
  function(data,
           window = "square",
           window.length = NULL) {
    data <- data.frame(data)
    ow <-
      spatstat.geom::owin(xrange = range(data$x), yrange = range(data$y))

    if (window == "convex") {
      p <- spatstat.geom::ppp(data$x, data$y, ow)
      ow <- spatstat.geom::convexhull(p)
    }
    if (window == "concave") {
      cli::cli_inform("Concave windows are temperamental. Try choosing values of window.length > and < 1 if you have problems.") # nolint
      if (is.null(window.length)) {
        window.length <- (max(data$x) - min(data$x)) / 20
      } else {
        window.length <- (max(data$x) - min(data$x)) / 20 * window.length
      }
      dist <- (max(data$x) - min(data$x)) / (length(data$x))
      bigDat <-
        do.call(
          "rbind",
          lapply(as.list(as.data.frame(t(data[, c("x", "y")]))), function(x) {
            cbind(
              x[1] + c(0, 1, 0, -1, -1, 0, 1, -1, 1) * dist,
              x[2] + c(0, 1, 1, 1, -1, -1, -1, 0, 0) * dist
            )
          })
        )
      ch <-
        concaveman::concaveman(bigDat,
          length_threshold = window.length,
          concavity = 1
        )
      poly <- as.data.frame(ch[nrow(ch):1, ]) # nolint
      colnames(poly) <- c("x", "y")
      ow <-
        spatstat.geom::owin(
          xrange = range(poly$x),
          yrange = range(poly$y),
          poly = poly
        )
    }
    ow
  }




#' @importFrom spatstat.explore density.ppp
#' @importFrom spatstat.geom closepairs nearest.valid.pixel area ppp
#' @importFrom tidyr pivot_longer
#' @importFrom dplyr left_join
#' @importFrom rlang .data
inhomLPair <- function(data,
                       Rs = c(20, 50, 100),
                       sigma = NULL,
                       window = "convex",
                       window.length = NULL,
                       minLambda = 0.05,
                       from = NULL,
                       to = NULL,
                       edgeCorrect = TRUE,
                       includeZeroCells = TRUE) {
  ow <- makeWindow(data, window, window.length)


  X <-
    spatstat.geom::ppp(
      x = data$x,
      y = data$y,
      window = ow,
      marks = data$cellType,
      check = FALSE
    )

  if (is.null(Rs)) {
    Rs <- c(20, 50, 100)
  }

  maxR <- min(ow$xrange[2] - ow$xrange[1], ow$yrange[2] - ow$yrange[1]) / 2.01
  Rs <- unique(pmin(c(0, sort(Rs)), maxR))

  if (!is.null(sigma)) {
    den <- spatstat.explore::density.ppp(X, sigma = sigma)
    den <- den / mean(den)
    den$v <- pmax(den$v, minLambda)
  }

  if (is.null(from)) from <- levels(data$cellType)
  if (is.null(to)) to <- levels(data$cellType)

  use <- data$cellType %in% c(from, to)
  if (all(!use)) {
    return(NA)
  }
  data <- data[use, ]
  X <- X[use, ]


  p <- spatstat.geom::closepairs(X, max(Rs), what = "ijd", distinct = FALSE)

  p$j <- data$cellID[p$j]
  p$i <- data$cellID[p$i]

  cT <- data$cellType
  names(cT) <- data$cellID

  p$d <- cut(p$d, Rs, labels = Rs[-1], include.lowest = TRUE)

  # inhom density
  p$wt <- rep(1, length(p$d))
  if (!is.null(sigma)) {
    np <- spatstat.geom::nearest.valid.pixel(X$x, X$y, den)
    w <- den$v[cbind(np$row, np$col)]
    names(w) <- data$cellID
    p$wt <- 1 / w[p$j] * mean(w)
    rm(np)
  }


  lam <- table(data$cellType) / spatstat.geom::area(X)

  p$cellTypeJ <- cT[p$j]
  p$cellTypeI <- cT[p$i]
  p$i <- factor(p$i, levels = data$cellID)


  if (edgeCorrect) {
    rList <- sapply(Rs[-1], function(x) {
      p2 <- p
      edge <- borderEdge(X, x)
      edge <- as.data.frame(edge)
      colnames(edge) <- x
      edge$i <- data$cellID
      edge <- tidyr::pivot_longer(edge, -.data$i, names_to = "d")
      p2 <- as.data.frame(p2)
      p2 <- p2[as.numeric(as.character(p2$d)) <= x, ]
      p2$d <- as.character(x)
      
      p2$i <- as.character(p2$i)
      p2$d <- as.character(p2$d)
      edge$i <- as.character(edge$i)
      edge$d <- as.character(edge$d)
      
      p2 <- dplyr::left_join(p2, edge, c("i", "d"))
      p2$d <- factor(p2$d, levels = x)
      p2 <- p2[p2$i != p2$j, ]
      use <- p2$cellTypeI %in% from & p2$cellTypeJ %in% to
      p2 <- p2[use, ]
      inhomL(p2, lam, X, x)
    }, simplify = FALSE)
    # browser()
    r <- do.call("rbind", rList)

    r <- dplyr::group_by(r, cellTypeI, cellTypeJ)
    r <- dplyr::summarise(r, wt = mean(wt))
  } else {
    p <- as.data.frame(p)
    p$value <- 1

    p$d <- factor(p$d, levels = Rs[-1])

    p <- p[p$i != p$j, ]

    use <- p$cellTypeI %in% from & p$cellTypeJ %in% to
    p <- p[use, ]

    r <- inhomL(p, lam, X, Rs)
  }




  wt <- r$wt
  names(wt) <- paste(r$cellTypeI, r$cellTypeJ, sep = "__")

  m1 <- rep(from, times = length(to))
  m2 <- rep(to, each = length(from))
  labels <- paste(m1, m2, sep = "__")

  assoc <- rep(-sum(Rs), length(labels))
  names(assoc) <- labels
  if (!includeZeroCells) assoc[!(m1 %in% X$marks & m2 %in% X$marks)] <- NA
  assoc[names(wt)] <- wt
  names(assoc) <- labels

  assoc
}


#' @importFrom data.table as.data.table setkey CJ .SD ":="
#' @importFrom spatstat.geom area
inhomL <-
  function(p, lam, X, Rs) {
    r <- data.table::as.data.table(p)
    r$wt <- r$wt / r$value / as.numeric(lam[r$cellTypeJ]) / as.numeric(lam[r$cellTypeI]) / spatstat.geom::area(X) # nolint
    r <- r[, j := NULL] # nolint
    r <- r[, value := NULL] # nolint
    r <- r[, i := NULL] # nolint
    data.table::setkey(r, d, cellTypeI, cellTypeJ) # nolint
    r <- r[data.table::CJ(d, cellTypeI, cellTypeJ, unique = TRUE)][, lapply(.SD, sum), by = .(d, cellTypeI, cellTypeJ)][is.na(wt), wt := 0] # nolint
    r <- r[, wt := cumsum(wt), by = list(cellTypeI, cellTypeJ)] # nolint
    r <- r[, list(wt = sum(sqrt(wt / pi))), by = .(cellTypeI, cellTypeJ)] # nolint
    r$wt <- r$wt - sum(Rs) # nolint

    r <- as.data.frame(r)

    r
  }




#' @importFrom spatstat.geom
#'     union.owin border inside.owin solapply intersect.owin area discs
borderEdge <- function(X, maxD) {
  W <- X$window
  bW <- spatstat.geom::union.owin(
    spatstat.geom::border(W, maxD, outside = FALSE),
    spatstat.geom::border(W, 2, outside = TRUE)
  )
  inB <- spatstat.geom::inside.owin(X$x, X$y, bW)
  e <- rep(1, X$n)
  if (any(inB)) {
    circs <- spatstat.geom::discs(X[inB], maxD, separate = TRUE)
    circs <- spatstat.geom::solapply(
      circs, spatstat.geom::intersect.owin, X$window
    )
    areas <- unlist(lapply(circs, spatstat.geom::area)) / (pi * maxD^2)
    e[inB] <- areas
  }

  e
}

#' @importFrom scam scam
#' @importFrom stats quantile
calcWeights <- function(rS, M1, M2, nCells, weightFactor) {
  count1 <- as.vector(nCells[, M1])
  count2 <- as.vector(nCells[, M2])
  rS <- as.vector(rS)
  toWeight <- !is.na(rS)
  resSqToWeight <- rS[toWeight]
  count1ToWeight <- count1[toWeight]
  count2ToWeight <- count2[toWeight]
  if (length(count1ToWeight) <= 20) {
    warning("A cell type pair is seen less than 20 times, not using weights for this pair.") # nolint
    weightFunction <- rep(1, length(count1))
    return(weightFunction)
  }
  weightFunction <- scam::scam(
    log10(resSqToWeight + 1) ~ s(log10(count1ToWeight + 1), bs = "mpd") + s(log10(count2ToWeight + 1), bs = "mpd") # nolint
  ) # , optimizer = "nlm.fd")

  z1 <- suppressWarnings(stats::predict(weightFunction, data.frame(
    count1ToWeight = as.numeric(count1),
    count2ToWeight = as.numeric(count2)
  )))
  w <- 1 / pmax(z1, stats::quantile(z1[z1 > 0.1], 0.01, na.rm = TRUE))
  w <- w / mean(w, na.rm = TRUE)
  w^weightFactor
}




#' @importFrom BiocParallel bpmapply
getWeightFunction <- function(
    pairwiseAssoc,
    nCells,
    m1,
    m2,
    BPPARAM,
    weights,
    weightsByPair,
    weightFactor) {
  if (!weights) {
    weightFunction <- rep(1, nrow(pairwiseAssoc) * ncol(pairwiseAssoc))
    pair <- rep(colnames(pairwiseAssoc), each = nrow(pairwiseAssoc))
    weightFunction <- split(weightFunction, pair)
    return(weightFunction)
  }


  resSq <- apply(pairwiseAssoc, 2, function(x) {
    if (sum(!is.na(x)) > 1) {
      if (stats::sd(x, na.rm = TRUE) > 0) {
        return((x - mean(x, na.rm = TRUE))^2)
      } else {
        return(rep(NA, length(x)))
      }
    } else {
      return(rep(NA, length(x)))
    }
  })

  if (weightsByPair) {
    weightFunction <- BiocParallel::bpmapply(
      calcWeights,
      rS = as.list(as.data.frame(resSq)), M1 = m1, M2 = m2, BPPARAM = BPPARAM,
      MoreArgs = list(nCells = nCells, weightFactor), SIMPLIFY = FALSE
    )
  } else {
    weightFunction <- calcWeights(m1, m2, rS = resSq, nCells, weightFactor)
    pair <- rep(colnames(pairwiseAssoc), each = nrow(pairwiseAssoc))
    weightFunction <- split(weightFunction, pair)
  }
  weightFunction[colnames(pairwiseAssoc)]
}





#' @importFrom SummarizedExperiment colData
#' @importFrom SpatialExperiment spatialCoords
#' @importFrom methods is
prepCellSummary <- function(
    cells, spatialCoords, cellType, imageID, bind = FALSE) {
  getCellSummary(cells, bind = bind)
}




#' Perform a simple wilcoxon-rank-sum test or t-test on the columns of a data
#' frame
#'
#' @param df A data.frame or SingleCellExperiment, SpatialExperiment
#' @param condition The condition of interest
#' @param type The type of test, "wilcox", "ttest" or "survival".
#' @param feature
#'     Can be used to calculate the proportions of this feature for each image
#' @param imageID The imageID's if presenting a SingleCellExperiment
#'
#' @return Proportions
#'
#'
#' @examples
#'
#' # Test for an association with long-duration diabetes
#' # This is clearly ignoring the repeated measures...
#' data("diabetesData")
#' diabetesData <- spicyR:::.format_data(
#'   diabetesData, "imageID", "cellType", c("x", "y"), FALSE
#' )
#' props <- getProp(diabetesData)
#' condition <- spicyR:::getImagePheno(diabetesData)$stage
#' names(condition) <- spicyR:::getImagePheno(diabetesData)$imageID
#' condition <- condition[condition %in% c("Long-duration", "Onset")]
#' test <- colTest(props[names(condition), ], condition)
#' @export
#' @importFrom SummarizedExperiment colData
#' @importFrom SingleCellExperiment SingleCellExperiment
#' @importFrom stats wilcox.test t.test
#' @importFrom S4Vectors as.data.frame
#' @importFrom ClassifyR colCoxTests
colTest <- function(
    df, 
    condition, 
    type = NULL, 
    feature = NULL, 
    imageID = "imageID") {
  if (is(df, "SingleCellExperiment") || is(df, "SpatialExperiment")) {
    if (is.null(feature)) stop("'feature' is still null")

    if (is.null(type) && length(condition) == 1) {
      type <- "ttest"
    } else if (is.null(type) && length(condition) == 2) {
      type <- "survival"
    } else if (is.null(type)) {
      stop("Invalid nuber of columns in condition. Must 1 or 2 (survival).")
    }

    x <- df@colData
    x <- x[, c(imageID, condition)]
    x <- unique(x)
    condition <- x[[condition]]
    names(condition) <- x[[imageID]]

    df <- getProp(df, imageID = imageID, feature = feature)
    condition <- condition[rownames(df)]
  } else {
    stopifnot(
      "Number of oberservation does not match between df and outcome" = {
        dim(df)[1] == dim(condition)[1]
      }
    )

    if (is.null(type) && (is.atomic(condition) || dim(condition)[2] == 1)) {
      type <- "ttest"
    } else if (is.null(type) && dim(condition)[2] == 2) {
      type <- "survival"
    } else if (is.null(type)) {
      stop("Invalid nuber of columns in condition. Must 1 or 2 (survival).")
    }
  }

  if (type == "survival") {
    test <- ClassifyR::colCoxTests(df, condition)
    test <- signif(test, 2)
    names(test)[names(test) == "p.value"] <- "pval"
  } else {
    test <- apply(df, 2, function(x) {
      if (type == "wilcox") {
        test <- stats::wilcox.test(x ~ condition)
      } else if (type == "ttest") {
        test <- stats::t.test(x ~ condition)
      }
      signif(c(test$estimate, tval = test$statistic, pval = test$p.value), 2)
    })
  }
  if (type != "survival") test <- as.data.frame(t(test))
  test$adjPval <- signif(stats::p.adjust(test$pval, "fdr"), 2)
  test$cluster <- rownames(test)
  test <- test[order(test$pval), ]
  test
}

#' Produces a dataframe showing L-function metric for each imageID entry.
#'
#' @param results
#'  Spicy test result obtained from spicy.
#' @param pairName
#'  A string specifying the pairwise interaction of interest. If NULL, all
#'  pairwise interactions are shown.
#'
#' @return A data.frame containing the colData related to the results.
#' @export
#'
#' @examples
#'
#' data(spicyTest)
#' df <- bind(spicyTest)
#'
#' @export
bind <- function(results,
                 pairName = NULL) {
  df <- data.frame(
    imageID = results$imageID,
    condition = results$condition
  )

  if (!is.null(results$subject)) {
    df <- cbind(df, subject = results$subject)
  }

  if (is.null(pairName)) {
    df <- cbind(df, do.call(cbind, results$pairwiseAssoc))
  } else {
    df <- cbind(df, results$pairwiseAssoc[[pairName]])
  }

  return(df)
}

test_that(
    "the output is equal to previous version",
    {
        original_result <- readRDS(
            system.file("testdata/original_result.rds", package = "spicyR")
        )
        expect_equal(
            suppressWarnings(spicy(diabetesData,
                condition = "stage", subject = "case",
                from = "Tc", to = "Th"
            )),
            original_result,
            tolerance = 0.01
        )
    }
)

#' Converts colPairs object into an abundance matrix based on number of nearby
#' interactions for every cell type.
#'
#' @param cells
#'   A SingleCellExperiment that contains objects in the colPairs slot.
#' @param colPair
#'   The name of the object in the colPairs slot for which the dataframe is
#'   constructed from.
#' @param imageID The image ID if using SingleCellExperiment.
#' @param cellType The cell type if using SingleCellExperiment.
#'
#' @return Matrix of abundances
#' @export
#'
#' @examples
#' data("diabetesData")
#' images <- c("A09", "A11", "A16", "A17")
#' diabetesData <- diabetesData[
#'   , SummarizedExperiment::colData(diabetesData)$imageID %in% images
#' ]
#'
#' diabetesData_SPE <- SpatialExperiment::SpatialExperiment(diabetesData,
#'   colData = SummarizedExperiment::colData(diabetesData)
#' )
#' SpatialExperiment::spatialCoords(diabetesData_SPE) <- data.frame(
#'   SummarizedExperiment::colData(diabetesData_SPE)$x,
#'   SummarizedExperiment::colData(diabetesData_SPE)$y
#' ) |>
#'   as.matrix()
#'
#' SpatialExperiment::spatialCoordsNames(diabetesData_SPE) <- c("x", "y")
#'
#' diabetesData_SPE <- imcRtools::buildSpatialGraph(diabetesData_SPE,
#'   img_id = "imageID",
#'   type = "knn",
#'   k = 20,
#'   coords = c("x", "y")
#' )
#'
#' pairAbundances <- convPairs(diabetesData_SPE,
#'   colPair = "knn_interaction_graph"
#' )
#'
#' @export
#' @importFrom SingleCellExperiment colPair colData
#' @importFrom tibble rownames_to_column column_to_rownames
convPairs <- function(cells,
                      colPair,
                      imageID = "imageID",
                      cellType = "cellType") {
  all_pairs <- SingleCellExperiment::colPair(cells, colPair) |>
    dplyr::as_tibble() |>
    # join the `from` cellType
    dplyr::left_join(
      SummarizedExperiment::colData(cells) |>
        dplyr::as_tibble() |>
        dplyr::select(imageID, cellType) |>
        tibble::rownames_to_column() |>
        dplyr::mutate(rowname = as.integer(rowname)),
      by = c("from" = "rowname")
    ) |>
    dplyr::rename(cellType_from = cellType) |>
    # join the `to` cellType
    dplyr::left_join(
      SummarizedExperiment::colData(cells) |>
        dplyr::as_tibble() |>
        dplyr::select(cellType) |>
        tibble::rownames_to_column() |>
        dplyr::mutate(rowname = as.integer(rowname)),
      by = c("to" = "rowname")
    ) |>
    dplyr::rename(cellType_to = cellType) |>
    # count the the number of `to` cellType associated with `from` cellTypes
    dplyr::mutate(one = 1L) |>
    dplyr::group_by(imageID, cellType_from, cellType_to) |>
    dplyr::summarise(n_close = sum(one), .groups = "drop") |>
    # join the total number of each cellType (within sample) into the dataframe
    dplyr::left_join(
      SummarizedExperiment::colData(cells) |>
        dplyr::as_tibble() |>
        dplyr::select(imageID, cellType) |>
        dplyr::group_by(imageID, cellType) |>
        dplyr::count(),
      by = c("cellType_from" = cellType, "imageID" = imageID)
    ) |>
    # calculate the association
    dplyr::mutate(association = n_close / n) |>
    dplyr::select(-n_close, -n) |>
    # wrangle the data into the correct format for spicy
    dplyr::mutate(
      test = paste(cellType_from, cellType_to, sep = "__")
    ) |>
    dplyr::select(imageID, test, association) |>
    tidyr::pivot_wider(
      names_from = test, values_from = association, values_fill = 0
    ) |>
    tibble::column_to_rownames(imageID)


  # Hot fix for spicy input when no cell type interactions exist for a pairwise
  # relation.
  vector <- cells$cellType |> unique()

  pairwise_vector <- c()

  for (i in vector) {
    for (j in vector) {
      pairwise_vector <- c(pairwise_vector, paste(i, j, sep = "__"))
    }
  }

  tmp <- dplyr::setdiff(pairwise_vector, colnames(all_pairs))
  df <- data.frame(matrix(0, nrow = nrow(all_pairs), ncol = length(tmp)))
  colnames(df) <- tmp

  all_pairs <- cbind(all_pairs, df)

  return(all_pairs)
}

#' Diabetes IMC data in SCE format.
#'
#' This is a subset of the Damond et al 2019 imaging mass cytometry dataset. The data 
#' contains cells in the pancreatic islets of individuals with early onset 
#' diabetes and healthy controls. 
#' The object contains single-cell data of 160 images from 8 subjects, 
#' with 20 images per subject.
#'
#' Converted into a SingleCellExperiment format.
#'
#' @format diabetesData_SCE a SingleCellExperiment object
#' @usage data("diabetesData")
#' @aliases 
#' diabetesData
"diabetesData"


#' Results from spicy for diabetesData
#'
#' Results from the call:
#' spicyTest <- spicy(diabetesData, 
#'                    condition = "condition", 
#'                    subject = "subject")
#'
#' @format spicyTest a spicy object
#' @usage data("spicyTest")
#' @aliases 
#' spicyTest
"spicyTest"

################################################################################
#
# Generics for spicy
#
################################################################################


#' A table of the significant results from spicy tests
#'
#' @param x The output from spicy.
#' @param coef Which coefficient to list.
#' @param n Extract the top n most significant pairs.
#' @param adj Which p-value adjustment method to use, argument for p.adjust().
#' @param cutoff A p-value threshold to extract significant pairs.
#' @param figures Round to `figures` significant figures.
#'
#' @return A data.frame
#'
#' @examples
#'
#' data(spicyTest)
#' topPairs(spicyTest)
#'
#' @aliases
#' topPairs,SpicyResults-method
#' topPairs
#' @rdname topPairs
#' @export
setGeneric("topPairs", function(x,
                                coef = NULL,
                                n = 10,
                                adj = "fdr",
                                cutoff = NULL,
                                figures = NULL) {
    standardGeneric("topPairs")
})
setMethod("topPairs", "SpicyResults", function(x,
                                               coef = NULL,
                                               n = 10,
                                               adj = "fdr",
                                               cutoff = NULL,
                                               figures = NULL) {
    if (!methods::is(x, "SpicyResults")) stop("x are not results from spicy")

    if (is.null(coef)) coef <- grep("condition", colnames(x$p.value))[1]

    if (methods::is(coef, "character") & !coef %in% colnames(x$p.value)) stop("coef not a column name")
    if (methods::is(coef, "numeric") & !coef %in% seq_len(ncol(x$p.value))) stop("coef not a column name")
    if (length(coef) > 1) warning("coef needs to be length 1, taking first entry.")
    useCondition <- coef[1]
    pval <- x$p.value[[useCondition]]
    adj.pvalue <- stats::p.adjust(pval, adj)

    comp <- x$comparison

    results <-
        data.frame(
            intercept = x$coefficient[, "(Intercept)"],
            coefficient = x$coefficient[, useCondition],
            p.value = pval,
            adj.pvalue = adj.pvalue,
            from = comp$from,
            to = comp$to
        )
    rownames(results) <- rownames(x$coefficient)
    if (length(results$p.value) > 0) {
        results <- results[order(results$p.value), ]

        if (is.null(cutoff) &&
            !is.null(n)) {
            results <- results[seq_len(pmin(n, nrow(results))), ]
        }
        if (is.null(n) &&
            !is.null(cutoff)) {
            results <- results[which(results$adj.pvalue <= cutoff), ]
        }
        if ((!is.null(n)) &&
            (!is.null(cutoff))) {
            results <- results[which(
                results$adj.pvalue <= cutoff & seq_len(nrow(results)) <= n
            ), ]
        }
    }
    if (!is.null(figures) && is.numeric(figures)) {
        results <- results |> dplyr::mutate(
            dplyr::across(is.numeric, signif, digits = figures)
        )
    }
    results
})

---
title: "Spatial Linear and Mixed-Effects Modelling with spicyR"
date: "`r BiocStyle::doc_date()`"
params:
  test: FALSE
author:
- name: Nicolas Canete
  affiliation:  
  - &WIMR Westmead Institute for Medical Research, University of Sydney, Australia
  email: nicolas.canete@sydney.edu.au
- name: Ellis Patrick
  affiliation:
  - &WIMR Westmead Institute for Medical Research, University of Sydney, Australia
  - School of Mathematics and Statistics, University of Sydney, Australia
- name: Alex Qin
  affiliation:
  - &WIMR Westmead Institute for Medical Research, University of Sydney, Australia
  - School of Mathematics and Statistics, University of Sydney, Australia
- name: Shreya Rao
  affiliation: 
    - &WIMR Westmead Institute for Medical Research, University of Sydney, Australia
    - School of Mathematics and Statistics, University of Sydney, Australia
package: "`r BiocStyle::pkg_ver('spicyR')`"
abstract: > 
  Perform linear and mixed-effects models to assess and visualise changes 
  in cell localisation across disease conditions.
vignette: >
  %\VignetteIndexEntry{"Spatial Linear and Mixed-Effects Modelling with spicy"}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
output: 
  BiocStyle::html_document:
    toc_float: true
    toc_depth: 3
bibliography: REFERENCES.bib
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  warning = FALSE,
  message = FALSE
)
library(BiocStyle)
```

# Installation

```{r, eval = FALSE}
if (!require("BiocManager")) {
  install.packages("BiocManager")
}
BiocManager::install("spicyR")
```

**Note:** There is a bug with the S4 method dispatch system, and `SingleCellExperiment` and `SpatialExperiment` objects are not correctly inheriting from `SummarizedExperiment.` One quick fix is to run the code below:


```{r warning=FALSE, message=FALSE}
# load required packages
library(SummarizedExperiment)

# code to work around the bug
methods::setClassUnion("ExpData", c("matrix", "dgCMatrix", "ExpressionSet",
                                    "SummarizedExperiment"))

library(spicyR)
library(ggplot2)
library(SpatialExperiment)
library(SpatialDatasets)
library(imcRtools)
library(dplyr)
library(survival)
```


# Overview

This guide provides step-by-step instructions on how to apply a linear model to multiple segmented and labelled images to assess how the localisation of different cell types changes across different disease conditions. 

# Example data

We use the @keren2018 breast cancer dataset to compare the spatial distribution of immune cells in individuals with different levels of tumour infiltration (cold and compartmentalised).

The data is stored as a `SpatialExperiment` object and contains single-cell spatial data from 41 images.

**Note:** There is a bug with the S4 method dispatch system, and `SingleCellExperiment` and `SpatialExperiment` objects are not correctly inheriting from `SummarizedExperiment.` One quick fix is to run the code below:


```{r warning=FALSE, message=FALSE}
kerenSPE <- SpatialDatasets::spe_Keren_2018()
```

The cell types in this dataset includes 11 immune cell types (double negative CD3 T cells, CD4 T cells, B cells, monocytes, macrophages, CD8 T cells, neutrophils, natural killer cells, dendritic cells, regulatory T cells), 2 structural cell types (endothelial, mesenchymal), 2 tumour cell types (keratin+ tumour, tumour) and one unidentified category.

# Linear modelling

To investigate changes in localisation between two different cell types, we 
measure the level of localisation between two cell types by modelling with the 
L-function. The L-function is a variance-stabilised K-function given by the equation

$$
\widehat{L_{ij}} (r) = \sqrt{\frac{\widehat{K_{ij}}(r)}{\pi}}
$$

with $\widehat{K_{ij}}$ defined as

$$
\widehat{K_{ij}} (r) = \frac{|W|}{n_i n_j} \sum_{n_i} \sum_{n_j} 1 \{d_{ij} \leq r \} e_{ij} (r)
$$

where $\widehat{K_{ij}}$ summarises the degree of co-localisation of cell type $j$ with cell type $i$, $n_i$ and $n_j$ are the number of cells of type $i$ and $j$, $|W|$ is the image area, $d_{ij}$ is the distance between two cells and $e_{ij} (r)$ is an edge correcting factor.

Specifically, the mean difference between the experimental function and the theoretical function is used as a measure for the level of localisation, defined as

$$
u = \sum_{r' = r_{\text{min}}}^{r_{\text{max}}} \widehat L_{ij, \text{Experimental}} (r') - \widehat L_{ij, \text{Poisson}} (r')
$$

where $u$ is the sum is taken over a discrete range of $r$ between $r_{\text{min}}$ and $r_{\text{max}}$. Differences of the statistic $u$ between two  conditions is modelled using a weighted linear model.

## Test for change in localisation for a specific pair of cells

Firstly, we can test whether one cell type tends to be more localised with another cell type in one condition compared to the other. This can be done using the `spicy()` function, where we specify the `condition` parameter.

In this example, we want to see whether or not neutrophils (`to`) tend to be found around CD8 T cells (`from`) in compartmentalised tumours compared to cold tumours. Given that there are 3 conditions, we can specify the desired conditions by setting the order of our `condition` factor. `spicy` will choose the first level of the factor as the base condition and the second level as the comparison condition. `spicy` will also naturally coerce the `condition` column into a factor if it is not already a factor. The column containing cell type annotations can be specified using the `cellType` argument. By default, `spicy` uses the column named `cellType` in the `SpatialExperiment` object.

```{r}
spicyTestPair <- spicy(
  kerenSPE,
  condition = "tumour_type",
  from = "CD8_T_cell",
  to = "Neutrophils"
)

topPairs(spicyTestPair)
```

We obtain a `spicy` object which details the results of the 
modelling performed. The `topPairs()` function can be used to obtain the associated coefficients and p-value.

As the `coefficient` in `spicyTestPair` is positive, we find that neutrophils are significantly more likely to be found near CD8 T cells in the compartmentalised tumours group compared to the cold tumour group.

## Test for change in localisation for all pairwise cell combinations

We can perform what we did above for all pairwise combinations of cell 
types by excluding the `from` and `to` parameters in `spicy()`.


```{r}
spicyTest <- spicy(
  kerenSPE,
  condition = "tumour_type"
)

topPairs(spicyTest)
```

Again, we obtain a `spicy` object which outlines the result of the linear
models performed for each pairwise combination of cell types.

We can also examine the L-function metrics of individual images by using the
convenient `bind()` function on our `spicyTest` results object.

```{r}
bind(spicyTest)[1:5, 1:5]
```


The results can be represented as a bubble plot using the `signifPlot()` function.

```{r}
signifPlot(
  spicyTest,
  breaks = c(-3, 3, 1),
  marksToPlot = c("Macrophages", "DC_or_Mono", "dn_T_CD3", "Neutrophils",
                  "CD8_T_cell", "Keratin_Tumour")
)
```

Here, we can observe that the most significant relationships occur between macrophages and double negative CD3 T cells, suggesting that the two cell types are far more dispersed in compartmentalised tumours compared to cold tumours.

To examine a specific cell type-cell type relationship in more detail, we can use `spicyBoxplot()` and specify either `from = "Macrophages"` and `to = "dn_T_CD3"` or `rank = 1`.

```{r}
spicyBoxPlot(results = spicyTest, 
             # from = "Macrophages",
             # to = "dn_T_CD3"
             rank = 1)
```

# Linear modelling for custom metrics

`spicyR` can also be applied to custom distance or abundance metrics. A kNN interactions graph can be generated with the function `buildSpatialGraph` from the `imcRtools` package. This generates a `colPairs` object inside of the `SpatialExperiment` object.

`spicyR` provides the function `convPairs` for converting a `colPairs` object into an abundance matrix by calculating the average number of nearby cells types for every cell type for a given `k`. For example, if there exists on average 5 neutrophils for every macrophage in image 1, the column `Neutrophil__Macrophage` would have a value of 5 for image 1.

```{r}
kerenSPE <- imcRtools::buildSpatialGraph(kerenSPE, 
                                         img_id = "imageID", 
                                         type = "knn", k = 20,
                                        coords = c("x", "y"))

pairAbundances <- convPairs(kerenSPE,
                  colPair = "knn_interaction_graph")

head(pairAbundances["B_cell__B_cell"])
```


The custom distance or abundance metrics can then be included in the analysis with the `alternateResult` parameter. The `Statial` package contains other custom distance metrics which can be used with `spicy`.

```{r}
spicyTestColPairs <- spicy(
  kerenSPE,
  condition = "tumour_type",
  alternateResult = pairAbundances,
  weights = FALSE
)

topPairs(spicyTestColPairs)
```


```{r}
signifPlot(
  spicyTestColPairs,
  breaks = c(-3, 3, 1),
  marksToPlot = c("Macrophages", "dn_T_CD3", "CD4_T_cell", 
                  "B_cell", "DC_or_Mono", "Neutrophils", "CD8_T_cell")
)
```

# Performing survival analysis 

`spicy` can also be used to perform survival analysis to asses whether changes in co-localisation between cell types are associated with survival probability. `spicy` requires the `SingleCellExperiment` object being used to contain a column called `survival` as a `Surv` object.

```{r}
kerenSPE$event = 1 - kerenSPE$Censored
kerenSPE$survival = Surv(kerenSPE$`Survival_days_capped*`, kerenSPE$event)
```

We can then perform survival analysis using the `spicy` function by specifying `condition = "survival"`. We can then access the corresponding coefficients and p-values by accessing the `survivalResults` slot in the `spicy` results object.

```{r}
# Running survival analysis
spicySurvival = spicy(kerenSPE,
                      condition = "survival")

# top 10 significant pairs
head(spicySurvival$survivalResults, 10)
```


# Accounting for tissue inhomogeneity

The `spicy` function can also account for tissue inhomogeneity to avoid false positives or negatives. This can be done by setting the `sigma = ` parameter within the spicy function. By default, `sigma` is set to `NULL`, and `spicy` assumes a homogeneous tissue structure. 

For example, when we examine the L-function for `Keratin_Tumour__Neutrophils` when `sigma = NULL` and `Rs = 100`, the value is positive, indicating attraction between the two cell types. 

```{r}
# filter SPE object to obtain image 24 data
kerenSubset = kerenSPE[, colData(kerenSPE)$imageID == "24"]

pairwiseAssoc = getPairwise(kerenSubset, 
                            sigma = NULL, 
                            Rs = 100) |>
  as.data.frame()

pairwiseAssoc[["Keratin_Tumour__Neutrophils"]]
```

When we specify `sigma = 20` and re-calculate the L-function, it indicates that there is no relationship between `Keratin_Tumour` and `Neutrophils`, i.e., there is no major attraction or dispersion, as it now takes into account tissue inhomogeneity.


```{r}
pairwiseAssoc = getPairwise(kerenSubset, 
                            sigma = 20, 
                            Rs = 100) |>
  as.data.frame()

pairwiseAssoc[["Keratin_Tumour__Neutrophils"]]
```

We can use the `plotImage` function to plot any pair of cell types for a specific image and visually inspect cellular relationships.

```{r}
plotImage(kerenSPE, "24", from = "Keratin_Tumour", to = "Neutrophils")
```

Plotting image 24 shows that the supposed co-localisation occurs due to the dense cluster of cells near the bottom of the image.

# Mixed effects modelling

`spicyR` supports mixed effects modelling when multiple images are obtained for each subject. In this case, `subject` is treated as a random effect and `condition` is treated as a fixed effect. To perform mixed effects modelling, we can specify the `subject` parameter in the `spicy` function.

```{r eval=FALSE}
spicyMixedTest <- spicy(
  diabetesData,
  condition = "stage",
  subject = "case"
)
```

# References


```{r}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spicy.R
\name{bind}
\alias{bind}
\title{Produces a dataframe showing L-function metric for each imageID entry.}
\usage{
bind(results, pairName = NULL)
}
\arguments{
\item{results}{Spicy test result obtained from spicy.}

\item{pairName}{A string specifying the pairwise interaction of interest. If NULL, all
pairwise interactions are shown.}
}
\value{
A data.frame containing the colData related to the results.
}
\description{
Produces a dataframe showing L-function metric for each imageID entry.
}
\examples{

data(spicyTest)
df <- bind(spicyTest)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spicyBoxPlot.R
\name{spicyBoxPlot}
\alias{spicyBoxPlot}
\title{Plots boxplot for a specified cell-cell relationship}
\usage{
spicyBoxPlot(results, from = NULL, to = NULL, rank = NULL)
}
\arguments{
\item{results}{Data frame obtained from spicy.}

\item{from}{Cell type which you would like to compare to the to cell type.}

\item{to}{Cell type which you would like to compare to the from cell type.}

\item{rank}{Ranking of cell type in terms of p-value, the smaller the p-value
the higher the rank.}
}
\value{
a ggplot2 boxplot
}
\description{
Plots boxplot for a specified cell-cell relationship
}
\examples{
data(spicyTest)

spicyBoxPlot(spicyTest,
             rank = 1)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/imageCrossPlot.R
\name{imageCrossPlot}
\alias{imageCrossPlot}
\title{Pairwise cross-plot of spatial associations}
\usage{
imageCrossPlot(
  result,
  image = NULL,
  colourGradient = c("#4575B4", "white", "#D73027"),
  marksToPlot = NULL,
  limits = NULL
)
}
\arguments{
\item{result}{A matrix or data frame produced by `getPairwise()` where
rownames are image IDs and colnames are concatenated marker pairs in the
form `"from__to"`. Must contain at least one row with name matching `image`.}

\item{image}{Character scalar. Which image (row in `result`) to plot. If
`NULL`, defaults to the first rowname of `result`.}

\item{colourGradient}{Character vector of length 3 giving the low, mid, and
high colours for a diverging palette used by `scale_colour_gradient2()`.
Default is `c("#4575B4", "white", "#D73027")`.}

\item{marksToPlot}{Optional character vector of marker names. If supplied, the
plot is restricted to rows and columns where both `from` and `to` are in
this set.}

\item{limits}{Numeric length-2 vector giving the lower and upper caps applied
to association values for colour mapping. Use `NULL` to avoid
clamping. Point sizes are clamped to `c(0, max(abs(limits)))`.}
}
\value{
A \pkg{ggplot2} object.
}
\description{
Builds a pairwise scatterplot of marker-marker associations for a single image
using the output of `getPairwise()`.
}
\details{
The function expects `colnames(result)` to contain `"__"` separating marker
names; otherwise an error is thrown. Values are transformed into two aesthetics:
\itemize{
  \item \code{value}: the (possibly clamped) signed association used for colour.
  \item \code{size}: the (possibly clamped) absolute association used for point size.
}
The colourbar shows three ticks at \code{lower}, \code{0}, and \code{upper}
with labels \code{"<lower"}, \code{"0"}, and \code{">upper"}.
}
\examples{
\dontrun{
# Minimal example (toy data)
set.seed(1)
mks <- c("A","B","C")
pairs <- as.vector(outer(mks, mks, paste, sep="__"))
res <- matrix(rnorm(length(pairs)*2), nrow = 2,
              dimnames = list(c("img1","img2"), pairs))

p <- imageCrossPlot(res, image = "img1",
                       colourGradient = c("#4575B4","white","#D73027"),
                       marksToPlot = c("A","B","C"),
                       limits = c(-3, 3))
print(p)
}

}
\seealso{
\code{\link{getPairwise}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spicy.R
\name{getProp}
\alias{getProp}
\title{Get proportions from a SummarizedExperiment.}
\usage{
getProp(cells, feature = "cellType", imageID = "imageID")
}
\arguments{
\item{cells}{A SingleCellExperiment, SpatialExperiment or data.frame.}

\item{feature}{The feature of interest}

\item{imageID}{The imageID's}
}
\value{
Proportions
}
\description{
Get proportions from a SummarizedExperiment.
}
\examples{
data("diabetesData")
prop <- getProp(diabetesData)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllClasses.R, R/spicy.R
\docType{class}
\name{SpicyResults-class}
\alias{SpicyResults-class}
\alias{SpicyResults,list,ANY-method}
\alias{spicy}
\alias{spicy,spicy-method}
\title{Performs spatial tests on spatial cytometry data.}
\usage{
spicy(
  cells,
  condition,
  subject = NULL,
  covariates = NULL,
  imageID = "imageID",
  cellType = "cellType",
  spatialCoords = c("x", "y"),
  r = NULL,
  sigma = NULL,
  from = NULL,
  to = NULL,
  alternateResult = NULL,
  cores = 1,
  minLambda = 0.05,
  weights = TRUE,
  weightsByPair = FALSE,
  weightFactor = 1,
  window = "convex",
  window.length = NULL,
  edgeCorrect = TRUE,
  includeZeroCells = FALSE,
  verbose = FALSE,
  BPPARAM = NULL,
  imageIDCol = imageID,
  cellTypeCol = cellType,
  spatialCoordCols = spatialCoords,
  nCores = cores,
  Rs = r,
  ...
)
}
\arguments{
\item{cells}{A SummarizedExperiment or data frame that contains at least the  variables
x and y, giving the location coordinates of each cell, and cellType.}

\item{condition}{A character specifying which column which contains the condition or `Surv` objects.}

\item{subject}{Vector of subject IDs corresponding to each image if cells is
a data frame.}

\item{covariates}{Vector of covariate names that should be included in the
mixed effects model as fixed effects.}

\item{imageID}{The name of the imageID column if using a SingleCellExperiment or SpatialExperiment.}

\item{cellType}{The name of the cellType column if using a SingleCellExperiment or SpatialExperiment.}

\item{spatialCoords}{The names of the spatialCoords column if using a SingleCellExperiment.}

\item{r}{A vector of the radii that the measures of association should be calculated over.}

\item{sigma}{A numeric variable used for scaling when fitting inhomogenous L-curves.}

\item{from}{vector of cell types which you would like to compare to the to vector.}

\item{to}{vector of cell types which you would like to compare to the from vector.}

\item{alternateResult}{A pairwise association statistic between each combination of celltypes in
each image.}

\item{cores}{Number of cores to use for parallel processing or a BiocParallel MulticoreParam or SerialParam object.}

\item{minLambda}{Minimum value density for scaling when fitting inhomogeneous L-curves.}

\item{weights}{logical indicating whether to include weights based on cell counts.}

\item{weightsByPair}{logical indicating whether weights should be calculated for each cell type
pair.}

\item{weightFactor}{numeric that controls the convexity of the weight function.}

\item{window}{Should the window around the regions be 'square', 'convex' or 'concave'.}

\item{window.length}{A tuning parameter for controlling the level of concavity when estimating concave windows.}

\item{edgeCorrect}{A logical indicating whether to perform edge correction.}

\item{includeZeroCells}{A logical indicating whether to include cells with zero counts in the pairwise association calculation.}

\item{verbose}{logical indicating whether to output messages.}

\item{BPPARAM}{\{DEPRECATED\} A BiocParallel MulticoreParam or SerialParam object.}

\item{imageIDCol}{\{DEPRECATED\} The name of the imageID column if using a SingleCellExperiment or SpatialExperiment.}

\item{cellTypeCol}{\{DEPRECATED\} The name of the cellType column if using a SingleCellExperiment or SpatialExperiment.}

\item{spatialCoordCols}{\{DEPRECATED\} The names of the spatialCoords column if using a SingleCellExperiment.}

\item{nCores}{\{DEPRECATED\} Number of cores to use for parallel processing or a BiocParallel MulticoreParam or SerialParam object.}

\item{Rs}{\{DEPRECATED\} A vector of the radii that the measures of association should be calculated over.}

\item{...}{Other options}
}
\value{
Data frame of p-values.
}
\description{
Performs spatial tests on spatial cytometry data.
}
\examples{
data("diabetesData")

# Test with random effect for patient on a pairwise combination of cell
# types.
spicy(diabetesData,
  condition = "stage", subject = "case",
  from = "Tc", to = "Th"
)

# Test all pairwise combinations of cell types without random effect of
# patient.
\dontrun{
spicyTest <- spicy(diabetesData, condition = "stage", subject = "case")
}

# Test all pairwise combination of cell types with random effect of patient.
\dontrun{
spicy(diabetesData, condition = "condition", subject = "subject")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{diabetesData}
\alias{diabetesData}
\title{Diabetes IMC data in SCE format.}
\format{
diabetesData_SCE a SingleCellExperiment object
}
\usage{
data("diabetesData")
}
\description{
This is a subset of the Damond et al 2019 imaging mass cytometry dataset. The data 
contains cells in the pancreatic islets of individuals with early onset 
diabetes and healthy controls. 
The object contains single-cell data of 160 images from 8 subjects, 
with 20 images per subject.
}
\details{
Converted into a SingleCellExperiment format.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/signifPlot.R
\name{signifPlot}
\alias{signifPlot}
\title{Plots result of signifPlot.}
\usage{
signifPlot(
  results,
  fdr = FALSE,
  type = "bubble",
  breaks = NULL,
  comparisonGroup = NULL,
  colours = c("#4575B4", "white", "#D73027"),
  marksToPlot = NULL,
  cutoff = 0.05,
  contextColours = NULL,
  contextLabels = waiver()
)
}
\arguments{
\item{results}{A spicy results object}

\item{fdr}{TRUE if FDR correction is used.}

\item{type}{Whether to make a bubble plot or heatmap. Note: For survival results a bubble plot will be used.}

\item{breaks}{Vector of 3 numbers giving breaks used in legend. The first
number is the minimum, the second is the maximum, the third is the
number of breaks.}

\item{comparisonGroup}{A string specifying the name of the outcome group to compare with the base group.}

\item{colours}{Vector of colours to use to colour legend.}

\item{marksToPlot}{Vector of marks to include in plot.}

\item{cutoff}{significance threshold for circles in bubble plot.}

\item{contextColours}{Used for \code{\link[Statial]{Kontextual}} results. A named list specifying the colours for each context.
By default the Tableau colour palette is used.}

\item{contextLabels}{Used for \code{\link[Statial]{Kontextual}} results. A named list to change the default labels for each context.}
}
\value{
a ggplot or pheatmap object
}
\description{
Plots result of signifPlot.
}
\examples{
data(spicyTest)

p <- signifPlot(spicyTest, breaks = c(-3, 3, 0.5))
# plot includes unicode characters, do not use default pdf device
ggplot2::ggsave(p, filename = tempfile(), device = cairo_pdf)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotImage.R
\name{plotImage}
\alias{plotImage}
\title{Plots an image with specified from and to cell types.}
\usage{
plotImage(
  cells,
  imageToPlot,
  from,
  to,
  imageID = "imageID",
  cellType = "cellType",
  spatialCoords = c("x", "y")
)
}
\arguments{
\item{cells}{A SummarizedExperiment object.}

\item{imageToPlot}{The ID of the image to be plotted.}

\item{from}{The "from" cell type.}

\item{to}{The "to" cell type.}

\item{imageID}{The name of the imageID column in the SummarizedExperiment object.}

\item{cellType}{The name of the cellType column in the SummarizedExperiment object.}

\item{spatialCoords}{The names of the spatialCoords column if using a SingleCellExperiment.}
}
\value{
A ggplot object.
}
\description{
Plots an image with specified from and to cell types.
}
\examples{
data("diabetesData")
plotImage(diabetesData, "A09", from = "acinar", to = "alpha")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convPairs.R
\name{convPairs}
\alias{convPairs}
\title{Converts colPairs object into an abundance matrix based on number of nearby
interactions for every cell type.}
\usage{
convPairs(cells, colPair, imageID = "imageID", cellType = "cellType")
}
\arguments{
\item{cells}{A SingleCellExperiment that contains objects in the colPairs slot.}

\item{colPair}{The name of the object in the colPairs slot for which the dataframe is
constructed from.}

\item{imageID}{The image ID if using SingleCellExperiment.}

\item{cellType}{The cell type if using SingleCellExperiment.}
}
\value{
Matrix of abundances
}
\description{
Converts colPairs object into an abundance matrix based on number of nearby
interactions for every cell type.
}
\examples{
data("diabetesData")
images <- c("A09", "A11", "A16", "A17")
diabetesData <- diabetesData[
  , SummarizedExperiment::colData(diabetesData)$imageID \%in\% images
]

diabetesData_SPE <- SpatialExperiment::SpatialExperiment(diabetesData,
  colData = SummarizedExperiment::colData(diabetesData)
)
SpatialExperiment::spatialCoords(diabetesData_SPE) <- data.frame(
  SummarizedExperiment::colData(diabetesData_SPE)$x,
  SummarizedExperiment::colData(diabetesData_SPE)$y
) |>
  as.matrix()

SpatialExperiment::spatialCoordsNames(diabetesData_SPE) <- c("x", "y")

diabetesData_SPE <- imcRtools::buildSpatialGraph(diabetesData_SPE,
  img_id = "imageID",
  type = "knn",
  k = 20,
  coords = c("x", "y")
)

pairAbundances <- convPairs(diabetesData_SPE,
  colPair = "knn_interaction_graph"
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{spicyTest}
\alias{spicyTest}
\title{Results from spicy for diabetesData}
\format{
spicyTest a spicy object
}
\usage{
data("spicyTest")
}
\description{
Results from the call:
spicyTest <- spicy(diabetesData, 
                   condition = "condition", 
                   subject = "subject")
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spicy.R
\name{getPairwise}
\alias{getPairwise}
\title{Get statistic from pairwise L curve of a single image.}
\usage{
getPairwise(
  cells,
  imageID = "imageID",
  cellType = "cellType",
  spatialCoords = c("x", "y"),
  r = NULL,
  sigma = NULL,
  from = NULL,
  to = NULL,
  cores = 1,
  minLambda = 0.05,
  window = "convex",
  window.length = NULL,
  edgeCorrect = TRUE,
  includeZeroCells = FALSE,
  BPPARAM = NULL,
  imageIDCol = imageID,
  cellTypeCol = cellType,
  spatialCoordCols = spatialCoords,
  nCores = cores,
  Rs = r
)
}
\arguments{
\item{cells}{A SummarizedExperiment that contains at least the
variables x and y, giving the location coordinates of each cell, and
cellType.}

\item{imageID}{The name of the imageID column if using a SingleCellExperiment or SpatialExperiment.}

\item{cellType}{The name of the cellType column if using a SingleCellExperiment or SpatialExperiment.}

\item{spatialCoords}{The names of the spatialCoords column if using a SingleCellExperiment.}

\item{r}{A vector of the radii that the measures of association should be calculated over.}

\item{sigma}{A numeric variable used for scaling when fitting inhomogenous L-curves.}

\item{from}{The 'from' cellType for generating the L curve.}

\item{to}{The 'to' cellType for generating the L curve.}

\item{cores}{Number of cores to use for parallel processing or a BiocParallel MulticoreParam or SerialParam object.}

\item{minLambda}{Minimum value density for scaling when fitting inhomogeneous L-curves.}

\item{window}{Should the window around the regions be 'square', 'convex' or 'concave'.}

\item{window.length}{A tuning parameter for controlling the level of concavity when estimating concave windows.}

\item{edgeCorrect}{A logical indicating whether to perform edge correction.}

\item{includeZeroCells}{A logical indicating whether to include cells with zero counts in the pairwise association.}

\item{BPPARAM}{\{DEPRECATED\} A BiocParallel MulticoreParam or SerialParam object.}

\item{imageIDCol}{\{DEPRECATED\} The name of the imageID column if using a SingleCellExperiment or SpatialExperiment.}

\item{cellTypeCol}{\{DEPRECATED\} The name of the cellType column if using a SingleCellExperiment or SpatialExperiment.}

\item{spatialCoordCols}{\{DEPRECATED\} The names of the spatialCoords column if using a SingleCellExperiment.}

\item{nCores}{\{DEPRECATED\} Number of cores to use for parallel processing or a BiocParallel MulticoreParam or SerialParam object.}

\item{Rs}{\{DEPRECATED\} A vector of the radii that the measures of association should be calculated over.
calculation.}
}
\value{
Statistic from pairwise L-curve of a single image.
}
\description{
Get statistic from pairwise L curve of a single image.
}
\examples{
data("diabetesData")
# Subset by imageID for fast example
selected_cells <- diabetesData[
  , SummarizedExperiment::colData(diabetesData)$imageID == "A09"
]
pairAssoc <- getPairwise(selected_cells)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R
\name{topPairs}
\alias{topPairs}
\alias{topPairs,SpicyResults-method}
\title{A table of the significant results from spicy tests}
\usage{
topPairs(x, coef = NULL, n = 10, adj = "fdr", cutoff = NULL, figures = NULL)
}
\arguments{
\item{x}{The output from spicy.}

\item{coef}{Which coefficient to list.}

\item{n}{Extract the top n most significant pairs.}

\item{adj}{Which p-value adjustment method to use, argument for p.adjust().}

\item{cutoff}{A p-value threshold to extract significant pairs.}

\item{figures}{Round to `figures` significant figures.}
}
\value{
A data.frame
}
\description{
A table of the significant results from spicy tests
}
\examples{

data(spicyTest)
topPairs(spicyTest)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spicy.R
\name{colTest}
\alias{colTest}
\title{Perform a simple wilcoxon-rank-sum test or t-test on the columns of a data
frame}
\usage{
colTest(df, condition, type = NULL, feature = NULL, imageID = "imageID")
}
\arguments{
\item{df}{A data.frame or SingleCellExperiment, SpatialExperiment}

\item{condition}{The condition of interest}

\item{type}{The type of test, "wilcox", "ttest" or "survival".}

\item{feature}{Can be used to calculate the proportions of this feature for each image}

\item{imageID}{The imageID's if presenting a SingleCellExperiment}
}
\value{
Proportions
}
\description{
Perform a simple wilcoxon-rank-sum test or t-test on the columns of a data
frame
}
\examples{

# Test for an association with long-duration diabetes
# This is clearly ignoring the repeated measures...
data("diabetesData")
diabetesData <- spicyR:::.format_data(
  diabetesData, "imageID", "cellType", c("x", "y"), FALSE
)
props <- getProp(diabetesData)
condition <- spicyR:::getImagePheno(diabetesData)$stage
names(condition) <- spicyR:::getImagePheno(diabetesData)$imageID
condition <- condition[condition \%in\% c("Long-duration", "Onset")]
test <- colTest(props[names(condition), ], condition)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/signifPlot.R
\name{survBubble}
\alias{survBubble}
\title{Plots survival results from spicy.}
\usage{
survBubble(
  result,
  fdr = FALSE,
  cutoff = 0.05,
  colourGradient = c("#4575B4", "white", "#D73027"),
  marksToPlot = NULL,
  contextColours = NULL,
  contextLabels = waiver()
)
}
\arguments{
\item{result}{A spicyResults object that contains survival results.}

\item{fdr}{TRUE if FDR correction is used.}

\item{cutoff}{Significance threshold for circles in bubble plot.}

\item{colourGradient}{A vector of colours, used to define the low, medium, and high values for the colour scale.}

\item{marksToPlot}{Vector of marks to include in bubble plot.}

\item{contextColours}{Used for \code{\link[Statial]{Kontextual}} results. A named list specifying the colours for each context.
By default the Tableau colour palette is used.}

\item{contextLabels}{Used for \code{\link[Statial]{Kontextual}} results. A named list to change the default labels for each context.}
}
\value{
A ggplot object.
}
\description{
Plots survival results from spicy.
}
